#!/usr/bin/env python

"""
Demonstration on how to grab the backend after some Easyviz commands
and add some backend specific commands. The example is based on the
legend_demo.py file from Matplotlib.
"""

from scitools.std import *

a = linspace(0,3,150)
b = linspace(0,3,150)
c = exp(a)
d = c.tolist()
d.reverse()
d = array(d)

setp(interactive=False)
plot(a,c,'k--',a,d,'k:',a,c+d,'k')
legend('Model length', 'Data length', 'Total message length')
ylim([-1,20])
xlabel('Model complexity --->')
ylabel('Message length --->')
title('Minimum Message Length')
show()

print "Let us now tune the plot by grabbing the backend."
raw_input('Press Return key to continue:')

# Grab the backend and fine tune the plot:
if backend == 'matplotlib':
    pyplot = get_backend()
    ax = pyplot.gca()
    # Remove tick labels:
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    # The arrows in the x and y labels does not look good. Here we use
    # some LaTeX arrows instead:
    ax.set_xlabel(r'Model complexity $\longrightarrow$')
    ax.set_ylabel(r'Message length $\longrightarrow$')
    # Change legend box location:
    l = ax.get_legend()
    l._loc = 9  # upper center (is there a better way to set location?)
    # Add shadow to legend box:
    l.shadow = True
    pyplot.draw()

elif backend == 'gnuplot':
    g = get_backend()
    # Remove tick labels:
    g('set xtics ("" 0, "" 0.5, "" 1, "" 1.5, "" 2, "" 2.5, "" 3)')
    g('set ytics ("" 0, "" 5, "" 10, "" 15, "" 20)')
    # Change location of legends:
    g('set key top center')
    # Create a box around the legends:
    g('set key box lt -1 lw 1 height 1 width 1')
    # Make the axes frame nice and thick:
    g('set border 1+2+4+8+16 linetype -1 linewidth 2')
    g.replot()

elif backend == 'veusz': 
    g = get_backend()
    # Remove tick labels:
    g.Set('/page1/graph1/x/TickLabels/hide', True)
    g.Set('/page1/graph1/y/TickLabels/hide', True)
    # Turn off minor tick marks:
    g.Set('/page1/graph1/x/MinorTicks/hide', True)
    g.Set('/page1/graph1/y/MinorTicks/hide', True)
    # Change legend box location:
    g.Set('/page1/graph1/key1/horzPosn', 'centre')
    g.Set('/page1/graph1/key1/vertPosn', 'top')
    # Adjust the placement of the title:
    g.Set('/page1/graph1/label1/xPos', 0.3)
    g.Set('/page1/graph1/label1/yPos', 1.02)
    g.window.showNormal()
    # We could also have done the changes interactively by saving the
    # session to a native Veusz file (.vsz) and open the file in Veusz:
    #save('tmp1.vsz')
    #os.system('veusz tmp1.vsz')

elif backend == 'grace':
    # In Grace we can do the changes interactively in the GUI.
    pass

elif backend == 'matlab':
    g = get_backend()
    # Remove tick labels:
    g.set_(g.gca(), 'XTickLabel', [], 'YTickLabel', [], nout=0)
    # Change legend location:
    h = g.findobj('Tag', 'legend', nout=1)  # find the legend handle
    g.set_(h, 'Location', 'North', nout=0)

elif backend == 'matlab2':
    g = get_backend()
    s = get_script()
    # Remove tick labels:
    s += "set(gca, 'XTickLabel', [], 'YTickLabel', [])\n"
    # Change legend location:
    s += "set(findobj('Tag', 'legend'), 'Location', 'North')\n"
    # Save the Matlab commands to a Matlab .m file:
    save('tmp1.m')
    # We can now open this file in Matlab (if Matlab is available):
    #os.system('matlab -r tmp1')

elif backend == 'pyx':
    # There is currently no way to change properties in the PyX backend
    # after having made the plot.
    pass

elif backend == 'blt':
    pass

print "Now we store a hardcopy of the tuned plot."
raw_input('Press Return key to continue:')

#hardcopy('grab_backend1.eps')  # Will destroy all changes!
#hardcopy('grab_backend1.png')  # Will destroy all changes!

# We have made changes in the backend directly and a call to hardcopy in
# Easyviz would normally destroy all these changes. However, setting the
# keyword argument replot to False in the hardcopy function will make sure
# that the changes are not destroyed:
if backend == 'gnuplot':
    # Unfortunately, setting replot to False does not currently work in the
    # Gnuplot backend. So, in this case we must use the hardcopy command from
    # the Gnuplot.py module:
    g.hardcopy(filename='tmp_grab_backend1.eps', terminal='postscript', mode='eps')
    g.hardcopy(filename='tmp_grab_backend1.png', terminal='png')
else:
    savefig('grab_backend1.eps', replot=False)
    if not backend == 'pyx':
        savefig('grab_backend1.png', replot=False)



"""
Functions for checking compatibility of data structures,
right type of data etc.

Example: given a program "check.py"::

    from numpy import linspace
    q1 = linspace(0, 1, 5)
    try:
        right_length(q1, 2)
    except Exception, e:
        print e.__class__.__name__, e
    try:
        right_size1(q1, (2,2))
    except Exception, e:
        print e.__class__.__name__, e
    try:    
        q2 = linspace(0, 1, 6)
    except Exception, e:
        print e.__class__.__name__, e
    try:
        right_size2(q1, q2)
    except Exception, e:
        print e.__class__.__name__, e
    try:
        right_type(q2, list)
    except Exception, e:
        print e.__class__.__name__, e
    try:
        wrong_type(q1)
    except Exception, e:
        print e.__class__.__name__, e

Here is the output (each call triggers the exception)::

    ValueError file "check.py", line 5, main program
    q1 has length 5, which is not compatible with assumed length 2
    ValueError file "check.py", line 9, main program
    q1 has size (5,), which is not compatible with assumed size (2, 2)
    ValueError file "check.py", line 17, main program
    q1 has size (5,), which is not compatible with size (6,) of q2
    TypeError file "check.py", line 21, main program
    q2 is of type ndarray, while we expected <type 'list'>
    TypeError q1 is of wrong type (ndarray).

"""

import types, re, inspect

__all__ = ['right_length', 'right_size1', 'right_size2',
           'get_type', 'right_type', 'wrong_type']

def get_argname_in_call(call_name, arg_no):
    """
    Utility function for extracting the name of an argument in a function
    call.

    Caveat: the function call is obtained as a string, and the arguments
    obtained by splitting that string with respect to comma.
    This can easily lead to errors, e.g., 'myfunc(a, (list,tuple), b)'
    will lead to the arguments 'a', '(list', 'tuple)', and 'b'.
    'b' is not obtained as argument 3.

    =========  ======================================================    
    call_name  name of the function in the call
    arg_no     argument number to extract (1, 2, ...).
    return     name of the argument and a string ("where") describing
               the original call's filename, line number and function.
    =========  ======================================================    
    """
    
    stack = inspect.stack()
    #import pprint; pprint.pprint(stack)
    #print 'frameinfo:\n', inspect.getframeinfo(stack[3][0])

    # call from some function:
    filename, line_number, func_name, func_call = stack[2][1:5]
    if func_name == '?':
        func_name = 'main program'
    if func_call is None:
        return 'variable', \
               '(info on variable names are missing since the call was made from an interpreter)'

    func_call = func_call[0]
    where = 'file "%s", line %d, %s' % (filename, line_number, func_name)
    pattern = r'%s\s*\((.*)\)' % call_name
    m = re.search(pattern, func_call)
    if m:
        args = m.group(1)
        # assume comma is separator (will not work if one of the args
        # is a tuple, dict or list explicitly listed with its value
        # in the call...)
        argname = args.split(',')[arg_no-1].strip()
        return argname, where
    else:
        raise ValueError('pattern="%s" does not match "%s"' % \
                         (pattern, func_call))


def right_length(a, length):
    """
    Check that len(a) == length.

    =========  ======================================================    
    a          any variable for which len(a) is meaningful
    length     the expected length of a (integer).
    =========  ======================================================    
    """
    if len(a) != length:
        a_name, where = get_argname_in_call('right_length', 1)
        raise ValueError(
            '%s\n%s has length %s, which is not compatible with '\
            'assumed length %s' % (where, a_name, len(a), length))
    else:
        return True

def right_size1(a, shape):
    """
    Check that a has correct shape.

    =========  ======================================================    
    a          NumPy array
    shape      the expected shape of a (int or tuple)
    =========  ======================================================    
    """
    if not hasattr(a, 'shape'):
        raise TypeError('%s is %s and not a NumPy array' % \
                        (a_name, type(a)))
    if isinstance(shape, int):
        shape = (shape,)  # wrap in tuple
    if a.shape != shape:
        a_name, where = get_argname_in_call('right_size1', 1)
        raise ValueError(
            '%s\n%s has size %s, which is not compatible with assumed size %s' \
            % (where, a_name, a.shape, shape))
    else:
        return True

def right_size2(a1, a2):
    """
    Check that a1 and a2 have equal shapes.
    a1 and a2 are NumPy arrays.
    """
    if hasattr(a1, 'shape') and hasattr(a2, 'shape'):
        pass # ok, a1 and a2 are NumPy arrays
    else:
        raise TypeError('%s is %s and %s is %s - both must be NumPy arrays' \
                        % (a1_name, type(a1), a2_name, type(a2)))
    if a1.shape != a2.shape:
        a1_name, where = get_argname_in_call('right_size2', 1)
        a2_name, where = get_argname_in_call('right_size2', 2)
        raise ValueError(
            '%s\n%s has size %s, which is not compatible with size %s of %s' \
            % (where, a1_name, a1.shape, a2.shape, a2_name))
    else:
        return True


def get_type(a):
    """
    Extract a's type. First, try to extract a's class name. If this
    is unsuccessful, return type(a).

    The session below illustrates differences between type(a) nad
    get_type(a) for standard classes, instances of user-defined classes,
    and class objects (new style and classic classes).

    >>> # standard built-in types:
    >>> c = [1,2,3]
    >>> d = 'my string'
    >>> type(c)
    <type 'list'>
    >>> get_type(c)
    'list'
    >>> type(d)
    <type 'str'>
    >>> get_type(d)
    'str'
    >>> 
    >>> # user-defined classes and instances:
    >>> class A:          # classic class
    ...     pass
    ... 
    >>> class B(object):  # new style class
    ...     pass
    ... 
    >>> a = A()
    >>> type(a)
    <type 'instance'>
    >>> get_type(a)
    'A'
    >>> 
    >>> b = B()
    >>> type(b)
    <class '__main__.B'>
    >>> get_type(b)
    'B'
    >>>
    >>> # class objects A and B:
    >>> type(A)
    <type 'classobj'>
    >>> get_type(A)
    'classobj (i.e. a class object)'
    >>> type(B)
    <type 'type'>
    >>> get_type(B)
    'type (i.e. a class object)'
    >>> 
    
    """
    try:
        # try to get a's class name (if possible)
        tp = a.__class__.__name__
        if tp == 'type':  # new style class object?
            # add some explanation (can get output "a is of type type")
            tp += ' (i.e. a class object)'
    except:
        # rely on the type (would be less informative if a is instance)
        tp = str(type(a))
        # if a is a classic class object, tp is "<type 'classobj'>"
        # which we translate into something more readable:
        if tp == "<type 'classobj'>":
            tp = "classobj (i.e. a class object)"
    return tp

global message
message = ''

def right_type(a, expected_types, raise_exception=True):
    """
    Check that variable a is of the type(s) specified by expected_types,
    which is a list/tuple of built-in types (class names), user-defined
    class names, or one of the functions callable or numpy.iterable.

    return: True if the type is right. Otherwise, raise a TypeError exception
    if raise_exception is True. If this value is False,
    return False and place a message string explaining what is wrong
    in the module variable errorcheck.message.
    """
    if not isinstance(expected_types, (list,tuple)):
        expected_types = [expected_types]  # wrap in list if just single type
    t = get_type(a)
    if 'a class object' in t:
        a_is_class_def = True
    else:
        a_is_class_def = False

    import numpy
    match = False
    for tp in expected_types:
        if tp is callable or tp is numpy.iterable:
            if tp(a):
                match = True
                break
        if a_is_class_def:
            # a is a class object
            #print 'a is class definition'
            if type(a) == types.ClassType:
                try:
                    if issubclass(a, tp):
                        match = True
                        break
                except:
                    pass
        else:
            try:
                #print 'testing isinstance(a, %s)' % str(tp)
                if isinstance(a, tp):
                    match = True
                    break
            except TypeError:
                if isinstance(a, type(tp)):
                    match = True
                    break
    if match:
        return True
    else:
        a_name, where = get_argname_in_call('right_type', 1)
        msg = '%s\n%s is of type %s, while we expected %s' % \
              (where, a_name, t, ' or '.join([str(e) for e in expected_types]))
        if raise_exception:
            raise TypeError(msg)
        else:
            global message
            message = msg
            return False
        

def wrong_type(a, comment=''):
    """
    Raise a TypeError exception expressing the type of variable a
    and that this type is wrong.

    a is any variable of any type, and comment is a text that can be
    appended to the exception string.
    """
    tp = get_type(a)
    a_name, where = get_argname_in_call('wrong_type', 1)
    raise TypeError('%s is of wrong type (%s). %s' % (a_name, tp, comment))


_SAFECODE = True
try:
    from globaldata import SAFECODE as _SAFECODE
except ImportError:
    pass
if not _SAFECODE:
    # define (efficient) empty check functions that overrides the
    # comprehensive and expensive functions above:
    for func in __all__:
        efficient_version = 'def %s(*args): return True' % func
        exec efficient_version

def _test():
    def myfunc(a, b, c):
        return a + b + c

    A = 1.1
    B = [1,2,3]
    class myclass:
        pass

    C = myclass()
    print right_type(myfunc, [callable, float, int])
    print right_type(A, [float, int])
    print right_type(C, [myclass])
    print right_type(myclass, [myclass])
    print right_type(B, tuple, raise_exception=False)
    print message

if __name__ == '__main__':
    _test()



from scitools.std import *
import time, glob, os

# Clean up old frames
for name in glob.glob('tmp_*.eps'):
    os.remove(name)

def f(x, m, s):
    return (1.0/(sqrt(2*pi)*s))*exp(-0.5*((x-m)/s)**2)

m = 0
s_start = 2
s_stop = 0.2
s_values = linspace(s_start, s_stop, 30)
x = linspace(m -3*s_start, m + 3*s_start, 1000)
# f is max for x=m; smaller s gives larger max value
max_f = f(m, m, s_stop)

# Show the movie, and make hardcopies of frames simulatenously
counter = 0
for s in s_values:
    y = f(x, m, s)
    plot(x, y, axis=[x[0], x[-1], -0.1, max_f],
         xlabel='x', ylabel='f', legend='s=%4.2f' % s,
         hardcopy='tmp_%04d.png' % counter)
    counter += 1
    #time.sleep(0.2)  # can insert a pause to control movie speed

# Make movie file the simplest possible way
movie('tmp_*.png')
import glob, os
print 'generated the file', glob.glob('movie.*')[0]
#os.remove(glob.glob('movie.*')[0])

# Make animated GIF movie in the file tmpmovie.gif
movie('tmp_*.png', encoder='convert', fps=2,
      output_file='tmpmovie.gif')

# Show movie (os.system runs an operating system command)
os.system('animate tmpmovie.gif &')

# Other formats
movie('tmp_*.png', encoder='html', fps=3,
      output_file='tmpmovie.html')  # play in HTML file

movie('tmp_*.png', encoder='ppmtompeg', fps=24,
      output_file='tmpmovie1.mpeg') # requires netpbm package

movie('tmp_*.png', encoder='ffmpeg', fps=4,
      output_file='tmpmovie1b.mpeg') # requires ffmpeg package

movie('tmp_*.png', encoder='ffmpeg', fps=4,
      output_file='tmpmovie1.avi') # requires ffmpeg package

movie('tmp_*.png', encoder='ffmpeg',
      output_file='tmpmovie1c.mpeg', vcodec='mpeg2video')


# mpeg_encode is not much available nowadays...
#movie('tmp_*.png', encoder='mpeg_encode', fps=24,
#      output_file='tmpmovie3.mpeg',
#      vcodec='mpeg2video', vbitrate=2400, qscale=4)




"""Manage a moving plot window for displaying very long time series."""
import time

class MovingPlotWindow:
    '''
    Make a plot window that follows the tip (end) of a very long
    time series.

    Example::

        def demo(I, k, dt, T, mode='continuous movement'):
            """
            Solve u' = -k**2*u, u(0)=I, u'(0)=0 by a finite difference
            method with time steps dt, from t=0 to t=T.
            """
            if dt > 2./k:
                print 'Unstable scheme'
            N = int(round(T/float(dt)))
            u = zeros(N+1)
            t = linspace(0, T, N+1)

            umin = -1.2*I
            umax = -umin
            period = 2*pi/k  # period of the oscillations
            window_width =
            plot_manager = MovingPlotWindow(
                             window_width, dt, yaxis=[umin, umax],
                             mode=mode)
            u[0] = I
            u[1] = u[0] - 0.5*dt**2*k**2*u[0]
            for n in range(1,N):
                u[n+1] = 2*u[n] - u[n-1] - dt**2*k**2*u[n]

                if plot_manager.plot(n):
                    s = plot_manager.first_index_in_plot
                    plot(t[s:n+2], u[s:n+2], 'r-',
                         t[s:n+2], I*cos(k*t)[s:n+2], 'b-',
                         axis=plot_manager.axis(),
                         title="Solution of u'' + k^2 u = 0 for t=%6.3f (mode: %s)" % (t[n+1], mode))
                plot_manager.update(n)
    '''

    def __init__(self, window_width, dt, yaxis=[-1,1],
                 mode='continuous movement',
                 pause=1.5):
        """
        ====================  ====================================
        Argument              Description
        ====================  ====================================
        window_width          tmax-tmin in a plot window
        dt                    time step (constant)
        yaxis                 extent of y axis
        mode                  method for moving the plot window,
                              see below.
        ====================  ====================================

        The mode parameter has three values:

          * ``'continuous movement'``:
            the plot window moves one time step for each plot.
          * ``'continuous drawing'``: the curves are drawn from left
            to right, one step at a time, and plot window jumps
            when the curve reaches the end.
          * ``'jumps'``: the curves are shown in a window for a time
            equal to the pause argument, then the axis jumps
            to a new time window of the same length


        See also the test block of the module for
        testing out the three different modes of this class.
        """
        self.taxis_length_in_steps = int(round(window_width/float(dt)))
        self.dt = dt
        self.mode = mode
        self.taxis_length = self.taxis_length_in_steps*self.dt
        self.first_index_in_plot = 0
        self.taxis_min = 0
        self.yaxis = yaxis
        self.pause = pause
        self.n_prev = 0

    def axis(self):
        """Return the axis limits as a list ``[xmin, xmax, ymin, ymax]``."""
        return [self.taxis_min, self.taxis_min + self.taxis_length,
                self.yaxis[0], self.yaxis[1]]

    def plot(self, n):
        """
        Return True if a plot is to be drawn at time step number ``n``,
        otherwise return False.
        """
        if self.mode == 'continuous movement':
            return True

        elif self.mode == 'continuous drawing':
            # Update self.first_index_in_plot and self.taxis_min
            # every self.taxis_length_in_steps time step
            if n % self.taxis_length_in_steps == 0:
                self.taxis_min += self.taxis_length
                self.first_index_in_plot += self.taxis_length_in_steps
            # Always plot the curves in continuous drawing
            return True
        else:
            # Plot every self.taxis_length_in_steps time step only
            if (n+1) % self.taxis_length_in_steps == 0:
                return True
            else:
                return False

    def update(self, n):
        """Update the plot manager (``MovingPlotWindow``) at time step ``n``."""
        if self.mode == 'continuous movement':
            if n > self.taxis_length_in_steps:
                self.taxis_min += self.dt
                self.first_index_in_plot += n - self.n_prev
            self.n_prev = n

        elif self.mode == 'jumps' and \
               (n+1) % self.taxis_length_in_steps == 0:
            # After plot is shown, sleep and then update plot
            # window parameters
            time.sleep(self.pause)
            self.taxis_min += self.taxis_length
            self.first_index_in_plot += self.taxis_length_in_steps

def _demo(I, k, dt, T, mode='continuous movement'):
    """
    Solve u' = -k**2*u, u(0)=I, u'(0)=0 by a finite difference
    method with time steps dt, from t=0 to t=T.
    """
    if dt > 2./k:
        print 'Unstable scheme'
    N = int(round(T/float(dt)))
    u = zeros(N+1)
    t = linspace(0, T, N+1)

    umin = -1.2*I
    umax = -umin
    period = 2*pi/k  # period of the oscillations
    plot_manager = MovingPlotWindow(8*period, dt, yaxis=[umin, umax],
                                    mode=mode)
    u[0] = I
    u[1] = u[0] - 0.5*dt**2*k**2*u[0]
    for n in range(1,N):
        u[n+1] = 2*u[n] - u[n-1] - dt**2*k**2*u[n]

        if plot_manager.plot(n):
            s = plot_manager.first_index_in_plot
            plot(t[s:n+2], u[s:n+2], 'r-',
                 t[s:n+2], I*cos(k*t)[s:n+2], 'b-',
                 axis=plot_manager.axis(),
                 title="Solution of u'' + k^2 u = 0 for t=%6.3f (mode: %s)" \
                 % (t[n+1], mode))
        plot_manager.update(n)

if __name__ == '__main__':
    from scitools.std import *
    T = 40
    _demo(1, pi, 0.1, T, mode='continuous movement')
    time.sleep(10)
    figure()
    _demo(1, pi, 0.1, T, mode='continuous drawing')
    time.sleep(10)
    figure()
    _demo(1, pi, 0.1, T, mode='jumps')
    time.sleep(10)




"""
This is a template file for writing new backends. It is a fully functional
backend, but no output is produced. One can specify this backend by

  python somefile.py --SCITOOLS_easyviz_backend template

or one can specify the backend in the SciTools configuration file
scitools.cfg under the [easyviz] section

  [easyviz]
  backend = template

and then

  from scitools.std import *

or if just easyviz is needed

  from scitools.easyviz import *

When writing a new backend, one can copy this file to xxx_.py (note the
underscore), where 'xxx' is the name of the new backend. Then replace all
instances of 'template' with 'xxx' and start implementing the class
methods below. The most important methods are figure, _replot, and hardcopy.

REQUIREMENTS:

<fill in requirements for backend>

"""

from __future__ import division

from common import *
from scitools.globaldata import DEBUG, VERBOSE
from scitools.misc import test_if_module_exists 

#test_if_module_exists('<package>', msg='You need to install the <package> package.', abort=False)
#import <package>


class TemplateBackend(BaseClass):
    def __init__(self):
        BaseClass.__init__(self)
        self._init()
        
    def _init(self, *args, **kwargs):
        """Perform initialization that is special for this backend."""
        
        self.figure(self.getp('curfig'))

        # conversion tables for format strings:
        self._markers = {
            '': None,   # no marker
            '.': None,  # dot
            'o': None,  # circle
            'x': None,  # cross
            '+': None,  # plus sign
            '*': None,  # asterisk
            's': None,  # square
            'd': None,  # diamond
            '^': None,  # triangle (up)
            'v': None,  # triangle (down)
            '<': None,  # triangle (left)
            '>': None,  # triangle (right)
            'p': None,  # pentagram
            'h': None,  # hexagram
            }
                        
        self._colors = {
            '': None,   # no color --> blue
            'r': None,  # red
            'g': None,  # green
            'b': None,  # blue
            'c': None,  # cyan
            'm': None,  # magenta
            'y': None,  # yellow
            'k': None,  # black
            'w': None,  # white
            }
        
        self._line_styles = {
            '': None,    # no line 
            '-': None,   # solid line
            ':': None,   # dotted line
            '-.': None,  # dash-dot line
            '--': None,  # dashed line
            }

        # convert table for colorbar location:
        self._colorbar_locations = {
            'North': None,
            'South': None,
            'East': None,
            'West': None,
            'NorthOutside': None,
            'SouthOutside': None,
            'EastOutside': None,
            'WestOutside': None,
            }

        if DEBUG:
            print "Setting backend standard variables"
            for disp in 'self._markers self._colors self._line_styles'.split():
                print disp, eval(disp)

    def _set_scale(self, ax):
        """Set linear or logarithmic (base 10) axis scale."""
        if DEBUG:
            print "Setting scales"
        scale = ax.getp('scale')
        if scale == 'loglog':
            # use logarithmic scale on both x- and y-axis
            pass
        elif scale == 'logx':
            # use logarithmic scale on x-axis and linear scale on y-axis
            pass
        elif scale == 'logy':
            # use linear scale on x-axis and logarithmic scale on y-axis
            pass
        elif scale == 'linear':
            # use linear scale on both x- and y-axis
            pass

    def _set_labels(self, ax):
        """Add text labels for x-, y-, and z-axis."""
        if DEBUG:
            print "Setting labels"
        xlabel = ax.getp('xlabel')
        ylabel = ax.getp('ylabel')
        zlabel = ax.getp('zlabel')
        if xlabel:
            # add a text label on x-axis
            pass
        if ylabel:
            # add a text label on y-axis
            pass
        if zlabel:
            # add a text label on z-axis
            pass
        
    def _set_title(self, ax):
        """Add a title at the top of the axis."""
        if DEBUG:
            print "Setting title"
        title = ax.getp('title')
        if title:
            pass  # set title
    
    def _set_limits(self, ax):
        """Set axis limits in x, y, and z direction."""
        if DEBUG:
            print "Setting axis limits"
        mode = ax.getp('mode')
        if mode == 'auto':
            # let plotting package set 'nice' axis limits in the x, y,
            # and z direction. If this is not automated in the plotting
            # package, one can use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            pass
        elif mode == 'manual':
            # (some) axis limits are frozen
            xmin = ax.getp('xmin')
            xmax = ax.getp('xmax')
            if xmin is not None and xmax is not None:
                # set x-axis limits
                pass
            else:
                # let plotting package set x-axis limits or use
                #xmin, xmax = ax.getp('xlim')
                pass

            ymin = ax.getp('ymin')
            ymax = ax.getp('ymax')
            if ymin is not None and ymax is not None:
                # set y-axis limits
                pass
            else:
                # let plotting package set y-axis limits or use
                #ymin, ymax = ax.getp('ylim')
                pass

            zmin = ax.getp('zmin')
            zmax = ax.getp('zmax')
            if zmin is not None and zmax is not None:
                # set z-axis limits
                pass
            else:
                # let plotting package set z-axis limits or use
                #zmin, zmax = ax.getp('zlim')
                pass
        elif mode == 'tight':
            # set the limits on the axis to the range of the data. If
            # this is not automated in the plotting package, one can
            # use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            pass
        elif mode == 'fill':
            # not sure about this
            pass

    def _set_position(self, ax):
        """Set axes position."""
        rect = ax.getp('viewport')
        if rect:
            # axes position is defined. In Matlab rect is defined as
            # [left,bottom,width,height], where the four parameters are
            # location values between 0 and 1 ((0,0) is the lower-left
            # corner and (1,1) is the upper-right corner).
            # NOTE: This can be different in the plotting package.
            pass

    def _set_daspect(self, ax):
        """Set data aspect ratio."""
        if ax.getp('daspectmode') == 'manual':
            dar = ax.getp('daspect')  # dar is a list (len(dar) is 3).
            pass
        else:
            # daspectmode is 'auto'. Plotting package handles data
            # aspect ratio automatically.
            pass
        
    def _set_axis_method(self, ax):
        method = ax.getp('method')
        if method == 'equal':
            # tick mark increments on the x-, y-, and z-axis should
            # be equal in size.
            pass
        elif method == 'image':
            # same effect as axis('equal') and axis('tight')
            pass
        elif method == 'square':
            # make the axis box square in size
            pass
        elif method == 'normal':
            # full size axis box
            pass
        elif method == 'vis3d':
            # freeze data aspect ratio when rotating 3D objects
            pass

    def _set_coordinate_system(self, ax):
        """
        Use either the default Cartesian coordinate system or a
        matrix coordinate system.
        """
        
        direction = ax.getp('direction')
        if direction == 'ij':
            # Use matrix coordinates. The origin of the coordinate
            # system is the upper-left corner. The i-axis should be
            # vertical and numbered from top to bottom, while the j-axis
            # should be horizontal and numbered from left to right.
            pass
        elif direction == 'xy':
            # use the default Cartesian axes form. The origin is at the
            # lower-left corner. The x-axis is vertical and numbered
            # from left to right, while the y-axis is vertical and
            # numbered from bottom to top.
            pass

    def _set_box(self, ax):
        """Turn box around axes boundary on or off."""
        if DEBUG:
            print "Setting box"
        if ax.getp('box'):
            # display box 
            pass
        else:
            # do not display box
            pass
        
    def _set_grid(self, ax):
        """Turn grid lines on or off."""
        if DEBUG:
            print "Setting grid"
        if ax.getp('grid'):
            # turn grid lines on
            pass
        else:
            # turn grid lines off
            pass

    def _set_hidden_line_removal(self, ax):
        """Turn on/off hidden line removal for meshes."""
        if DEBUG:
            print "Setting hidden line removal"
        if ax.getp('hidden'):
            # turn hidden line removal on
            pass
        else:
            # turn hidden line removal off
            pass

    def _set_colorbar(self, ax):
        """Add a colorbar to the axis."""
        if DEBUG:
            print "Setting colorbar"
        cbar = ax.getp('colorbar')
        if cbar.getp('visible'):
            # turn on colorbar
            cbar_title = cbar.getp('cbtitle')
            cbar_location = self._colorbar_locations[cbar.getp('cblocation')]
            # ...
        else:
            # turn off colorbar
            pass

    def _set_caxis(self, ax):
        """Set the color axis scale."""
        if DEBUG:
            print "Setting caxis"
        if ax.getp('caxismode') == 'manual':
            cmin, cmax = ax.getp('caxis')
            # NOTE: cmin and cmax might be None:
            if cmin is None or cmax is None:
                cmin, cmax = [0,1]
            # set color axis scaling according to cmin and cmax
            pass
        else:
            # use autoranging for color axis scale
            pass

    def _set_colormap(self, ax):
        """Set the colormap."""
        if DEBUG:
            print "Setting colormap"
        cmap = ax.getp('colormap')
        # cmap is plotting package dependent

    def _set_view(self, ax):
        """Set viewpoint specification."""
        if DEBUG:
            print "Setting view"
        cam = ax.getp('camera')
        view = cam.getp('view')
        if view == 2:
            # setup a default 2D view
            pass
        elif view == 3:
            az = cam.getp('azimuth')
            el = cam.getp('elevation')
            if az is None or el is None:
                # azimuth or elevation is not given. Set up a default
                # 3D view (az=-37.5 and el=30 is the default 3D view in
                # Matlab).
                pass
            else:
                # set a 3D view according to az and el
                pass
            
            if cam.getp('cammode') == 'manual':
                # for advanced camera handling:
                roll = cam.getp('camroll')
                zoom = cam.getp('camzoom')
                dolly = cam.getp('camdolly')
                target = cam.getp('camtarget')
                position = cam.getp('campos')
                up_vector = cam.getp('camup')
                view_angle = cam.getp('camva')
                projection = cam.getp('camproj')

    def _set_axis_props(self, ax):
        if DEBUG:
            print "Setting axis properties"
        self._set_title(ax)
        self._set_scale(ax)
        self._set_limits(ax)
        self._set_position(ax)
        self._set_axis_method(ax)
        self._set_daspect(ax)
        self._set_coordinate_system(ax)
        self._set_hidden_line_removal(ax)
        self._set_colorbar(ax)
        self._set_caxis(ax)
        self._set_colormap(ax)
        self._set_view(ax)
        if ax.getp('visible'):
            self._set_labels(ax)
            self._set_box(ax)
            self._set_grid(ax)
        else:
            # turn off all axis labeling, tickmarks, and background
            pass

    def _get_linespecs(self, item):
        """
        Return the line marker, line color, line style, and
        line width of the item.
        """
        
        marker = self._markers[item.getp('linemarker')]
        color = self._colors[item.getp('linecolor')]
        style = self._line_styles[item.getp('linetype')]
        width = item.getp('linewidth')
        return marker, color, style, width

    def _add_line(self, item):
        """Add a 2D or 3D curve to the scene."""
        if DEBUG:
            print "Adding a line"
        # get data:
        x = item.getp('xdata')
        y = item.getp('ydata')
        z = item.getp('zdata')
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        if z is not None:
            # zdata is given, add a 3D curve:
            pass
        else:
            # no zdata, add a 2D curve:
            pass

    def _add_surface(self, item, shading='faceted'):
        if DEBUG:
            print "Adding a surface"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field
        c = item.getp('cdata')  # pseudocolor data (can be None)
        
        contours = item.getp('contours')
        if contours:
            # the current item is produced by meshc or surfc and we
            # should therefore add contours at the bottom:
            self._add_contours(contours, placement='bottom')

        if item.getp('wireframe'):
            # wireframe mesh (as produced by mesh or meshc)
            pass
        else:
            # colored surface (as produced by surf, surfc, or pcolor)
            # use keyword argument shading to set the color shading mode
            pass

    def _add_contours(self, item, placement=None):
        # The placement keyword can be either None or 'bottom'. The
        # latter specifies that the contours should be placed at the
        # bottom (as in meshc or surfc).
        if DEBUG:
            print "Adding contours"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field

        filled = item.getp('filled')  # draw filled contour plot if True

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass

        location = item.getp('clocation')
        if location == 'surface':
            # place the contours at the corresponding z level (contour3)
            pass
        elif location == 'base':
            if placement == 'bottom':
                # place the contours at the bottom (as in meshc or surfc)
                pass
            else:
                # standard contour plot
                pass

        if item.getp('clabels'):
            # add labels on the contour curves
            pass
    
    def _add_vectors(self, item):
        if DEBUG:
            print "Adding vectors"
        # uncomment the following command if there is no support for
        # automatic scaling of vectors in the current plotting package:
        #item.scale_vectors()

        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # get line specifiactions (marker='.' means no marker):
        marker, color, style, width = self._get_linespecs(item)

        # scale the vectors according to this variable (scale=0 should
        # turn off automatic scaling):
        scale = item.getp('arrowscale')

        filled = item.getp('filledarrows') # draw filled arrows if True

        if z is not None and w is not None:
            # draw velocity vectors as arrows with components (u,v,w) at
            # points (x,y,z):
            pass
        else:
            # draw velocity vectors as arrows with components (u,v) at
            # points (x,y):
            pass

    def _add_streams(self, item):
        if DEBUG:
            print "Adding streams"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # starting positions for streams:
        sx, sy, sz = item.getp('startx'), item.getp('starty'), item.getp('startz')

        if item.getp('tubes'):
            # draw stream tubes from vector data (u,v,w) at points (x,y,z)
            n = item.getp('n') # no points along the circumference of the tube
            scale = item.getp('tubescale')
            pass
        elif item.getp('ribbons'):
            # draw stream ribbons from vector data (u,v,w) at points (x,y,z)
            width = item.getp('ribbonwidth')
            pass
        else:
            if z is not None and w is not None:
                # draw stream lines from vector data (u,v,w) at points (x,y,z)
                pass
            else:
                # draw stream lines from vector data (u,v) at points (x,y)
                pass
            pass

    def _add_isosurface(self, item):
        if DEBUG:
            print "Adding a isosurface"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume
        c = item.getp('cdata')  # pseudocolor data
        isovalue = item.getp('isovalue')

    def _add_slices(self, item):
        if DEBUG:
            print "Adding slices in a volume"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass
        pass

    def _add_contourslices(self, item):
        if DEBUG:
            print "Adding contours in slice planes"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels per plane
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass
        pass

    def _set_figure_size(self, fig):
        if DEBUG:
            print "Setting figure size"
        width, height = fig.getp('size')
        if width and height:
            # set figure width and height
            pass
        else:
            # use the default width and height in plotting package
            pass

    def figure(self, *args, **kwargs):
        # Extension of BaseClass.figure:
        # add a plotting package figure instance as fig._g and create a
        # link to it as self._g
        fig = BaseClass.figure(self, *args, **kwargs)
        try:
            fig._g
        except:
            # create plotting package figure and save figure instance
            # as fig._g
            if DEBUG:
                name = 'Fig ' + str(fig.getp('number'))
                print "creating figure %s in backend" % name

            class fig__:
                pass
            fig._g = fig__()
            
        self._g = fig._g  # link for faster access
        return fig

    def _replot(self):
        """Replot all axes and all plotitems in the backend."""
        # NOTE: only the current figure (gcf) is redrawn.
        if DEBUG:
            print "Doing replot in backend"
        
        fig = self.gcf()
        # reset the plotting package instance in fig._g now if needed
        
        self._set_figure_size(fig)
        
        nrows, ncolumns = fig.getp('axshape')
        for axnr, ax in fig.getp('axes').items():
            if nrows != 1 or ncolumns != 1:
                # create axes in tiled position
                # this is subplot(nrows,ncolumns,axnr)
                pass
            plotitems = ax.getp('plotitems')
            plotitems.sort(self._cmpPlotProperties)
            for item in plotitems:
                func = item.getp('function') # function that produced this item
                if isinstance(item, Line):
                    self._add_line(item)
                elif isinstance(item, Surface):
                    self._add_surface(item, shading=ax.getp('shading'))
                elif isinstance(item, Contours):
                    self._add_contours(item)
                elif isinstance(item, VelocityVectors):
                    self._add_vectors(item)
                elif isinstance(item, Streams):
                    self._add_streams(item)
                elif isinstance(item, Volume):
                    if func == 'isosurface':
                        self._add_isosurface(item)
                    elif func == 'slice_':
                        self._add_slices(item)
                    elif func == 'contourslice':
                        self._add_contourslices(item)
                legend = item.getp('legend')
                if legend:
                    # add legend to plot
                    pass

            self._set_axis_props(ax)
                    
        if self.getp('show'):
            # display plot on the screen
            if DEBUG:
                print "\nDumping plot data to screen\n"
                debug(self)
            pass

    def hardcopy(self, filename, **kwargs):
        """
        Supported extensions: <fill in extensions for this backend>
        """
        self.setp(**kwargs)
        color = self.getp('color')
        replot = kwargs.get('replot', True)
        if replot:
            self._replot()

        if DEBUG:
            print "Hardcopy to %s" % filename

    # reimplement color maps and other methods (if necessary) like clf,
    # closefig, and closefigs here.


    # Now we add the doc string from the methods in BaseClass to the
    # methods that are reimplemented in this backend:
    for cmd in BaseClass._matlab_like_cmds:
        if not '__' in cmd and hasattr(BaseClass, cmd):
            m1 = eval('BaseClass.%s' % cmd)
            try:
                m2 = eval('%s' % cmd)
            except NameError:
                pass
            else:
                if m1.__doc__ != m2.__doc__:
                    if m2.__doc__ is None:
                        m2.__doc__ = ""
                    m2.__doc__ = m1.__doc__ + m2.__doc__

    
plt = TemplateBackend()  # create backend instance
use(plt, globals())      # export public namespace of plt to globals()



"""
This module contains a class for managing efficiency/performance
experiments.
"""


class EfficiencyTable:
    """
    Manage the CPU times of efficiency experiments and make
    tabular reports with sorted results.

    >>> e = EfficiencyTable('some description of the experiments')
    >>> e.add('about an experiment', cpu_time)
    >>> e.add('about another experiment', cpu_time)
    >>> e.add('about a third experiment', cpu_time)
    >>> print e  # prints a sorted table with scaled CPU times
    >>> e += e2  # add experiments from EfficiencyTable e2 to e
    """
    def __init__(self, description, normalization_time=None):
        """
        @param description: a string acting as headline for this test.
        @param normalization_time: all CPU times will be divided by this value
        (if not set, the class will find the smallest (best) CPU
        time and divide all others by this value.

        The best_time parameter can also be set in the
        set_normalization_time method. The normalization time is not
        used before an instance is printed (str method).)
        """
        self.description = description
        self.experiments = {}  # key=description, value=[CPU-time1, CPU-time2, ]
        self._normalization_time = normalization_time

    def add(self, description, CPU_time, function_name=None):
        """
        Add the CPU time of an experiment, together with a description
        and an optional function_name (which is appended to the
        description string).
        """
        if function_name is not None:
            # include name of the tested function as part of the description:
            description = description + ' (%s)' % function_name
        if not description in self.experiments:
            self.experiments[description] = []
        self.experiments[description].append(CPU_time)

    def __iadd__(self, other):
        """
        Add results in other to present results.
        New items (descriptions) are simply registered, while
        identical items (descriptions) will have a list of CPU times,
        corresponding to the different EfficiencyTable instances.
        """
        self.description += '\n' + other.description
        for e in other.experiments:
            if e in self.experiments:
                # this experiment is already registered, add the lists of CPU times:
                self.experiments[e].extend(other.experiments[e])
            else:
                self.experiments[e] = other.experiments[e]
        return self
            
    def __add__(self, other):
        """As e += table (see __iadd__)."""
        e = EfficiencyTable(self.description)
        e.experiments = self.experiments.copy()
        e.__iadd__(other)
        return e

    def set_normalization_time(self, t):
        """
        Set the CPU time by which all other CPU times will be divided.
        By default, this is the maximum CPU time encountered in the data.
        """
        self._normalization_time = t
        
    def _reference_CPU_time(self, experiment_idx=0):
        if self._normalization_time is not None:
            # try first to see if there is an experiment with the
            # given normalization time, and if so, use the corresponding
            # description, otherwise use a dummy description:
            for description in self.experiments:
                if abs(self.experiments[description][experiment_idx] - \
                       self._normalization_time) < 1.0E-10:
                    return self._normalization_time, description
            # no experiment coincides with the given normalization time
            description = 'some external experiment'
            self.experiments[description] = [self._normalization_time]
            return self._normalization_time, description
            
                
        # no given normalization time, find best performance:
        # (only search among positive CPU times for an experiment with
        # index experiment_idx)
        best = 1.0E+20
        cpu_eps = 1.0E-9  # smallest reliable CPU time (but many repetitions
                          # may produce small time per call while the
                          # measurements are reliable)
        for description in self.experiments:
            cpu_time = self.experiments[description][experiment_idx]
            # drop counting very small (unreliable) or negative
            # (erroneous timings) CPU times:
            if cpu_time > cpu_eps:
                if cpu_time < best:
                    best = cpu_time
                    best_key = description

        if best == 1.0E+20:
            # did not find any CPU time > cpu_eps
            raise ValueError('too small CPU times (all less than %E)' % cpu_eps)
        return best, best_key
    
    def __str__(self):
        """
        Print out a sorted list (with respect to CPU times) of the experiments.
        In case of multiple CPU times per description of an experiment,
        the table is sorted with respect to the first CPU time entry of each
        multiple CPU times list. All CPU times are divided by a normalization
        time, which is given to the constructor or to the
        set_normalization_time method, or if not prescribed, this class
        finds the smallest reliable CPU time (neglecting very small
        CPU time).
        """
        # inv_dict is the inverse dictionary of self.experiments, i.e.,
        # CPU time is the key and the description is the valid.
        # Only the first CPU time entry is used.
        
        # inv_dict computation does not work if the CPU times are very
        # small (0.00 is the key of many), so we need to add a small
        # random number to very small CPU times
        import random, math
        inv_dict = {}  # inverse of self.experiments
        for k in self.experiments:
            CPU_time = self.experiments[k][0]
            if math.fabs(CPU_time) < 1.0E-7:
                CPU_time += 1.0E-14*random.random()
            if CPU_time in inv_dict:
                # this destroys the one-to-one mapping, perturb CPU_time:
                CPU_time *= 1.0 + 1.0E-3*random.random()
            self.experiments[k][0] = CPU_time
            inv_dict[CPU_time] = k
        # sort CPU-times:
        cpu_times0 = inv_dict.keys()
        cpu_times0.sort()
        s = '\n\n' + '*'*80 + '\n' + self.description + '\n' + '*'*80 + '\n'
        self.best, self.best_key = self._reference_CPU_time(0)
        s += 'reference CPU time based on the experiment\n   "%s"\nwith '\
             'CPU time:\n  %s\n\n' % \
             (self.best_key, str(self.experiments[self.best_key])[1:-1])

        max_length = max([len(string) for string in self.experiments.keys()])
        for cpu_time_key in cpu_times0:
            description = inv_dict[cpu_time_key]
            s += '%%-%ds | ' % max_length % description
            for cpu_time, ref_time in \
                    zip(self.experiments[description],
                        self.experiments[self.best_key]):
                nc = cpu_time/ref_time
                if abs(nc) > 9999.0:
                    s += '%10.1e' % nc
                else:
                    s += '%8.2f' % nc
            s += '\n'
        return s

def plot(filename):
    f = open(filename, 'r')
    lines = f.readlines()
    f.close()
    for i in range(len(lines)):
        if lines[i].find('CPU times') >= 0:
            start = i+3
            break
    counter = 1
    curves = {}
    labels = open('tmp_plot_labels', 'w')
    import math
    for line in lines[start:]:
        name, numbers = line.split('|')
        curves[name] = [float(x) for x in numbers.split()]
        for i in range(len(curves[name])):
            try:
                curves[name][i] = math.log10(curves[name][i])
            except ValueError:
                curves[name][i] = 0.0
        f = open('tmp_plot_%02d' % counter, 'w')
        for i in range(len(curves[name])):
            v = curves[name][i]
            if v > 0.0:
                f.write('%2d  %g\n' % (i+1, v))
        f.close()
        counter += 1
        labels.write('%2d: %s\n' % (counter, name))
    labels.close()
    # generate Gnuplot script:
    plotfiles = ['"tmp_plot_%02d" title "%d" with lines' % (i,i) \
                 for i in range(1,len(lines[start:])+1)]
    cmd = 'plot ' + ', '.join(plotfiles)
    f = open('tmp_plot.gnuplot', 'w')
    f.write("""
set xrange [0:%d]
%s
""" % (len(curves[name])+1, cmd))
    f.close()
                   
        
        
def _test(n):
    # how much does it cost to run an empty loop with
    # range, xrange and iseq?
    e = EfficiencyTable('Empty loops, loop length = %d' % n)
    import timeit
    t1 = timeit.Timer('for i in range(n): pass',
                      setup='n=%d' % n).timeit(5)
    e.add('for i in range(n): pass', t1)
    t2 = timeit.Timer('for i in xrange(n): pass',
                      setup='n=%d' % n).timeit(5)
    e.add('for i in xrange(n): pass', t2)
    t3 = timeit.Timer('for i in iseq(stop=n-1): pass',
                      setup='from scitools.numpyutils import iseq;' +
                      'n=%d' % n).timeit(5)
    e.add('for i in iseq(stop=n-1): pass', t3)
    print e
    
if __name__ == '__main__':
    import sys
    try:
        n = int(sys.argv[1])
    except:
        n = 100
    _test(n)



#!/usr/bin/env python

# Simple diagnostic tool for scitools.easyviz

import os, sys
import time, glob
import platform
import pprint
import signal
import ctypes
from subprocess import Popen, PIPE, STDOUT

# Taken from
# http://ivory.idyll.org/blog/mar-07/replacing-commands-with-subprocess
def get_status_output_errors(cmd, input=None, cwd=None, env=None):
    pipe = Popen(cmd, shell=True, cwd=cwd, env=env, stdout=PIPE, stderr=PIPE)

    output, errout = pipe.communicate(input=input)

    status = pipe.returncode

    return status, output, errout

def message(msg, diag_file=None, stdout=True):
    if stdout:
        print msg,
        sys.stdout.flush()
    if isinstance(diag_file, file) and not diag_file.closed:
        diag_file.write(msg)

def wait(str=None, prompt='Press return to show results...\n'):
    if str is not None:
        print str
    raw_input(prompt)

def wait(seconds=1):
    time.sleep(seconds)

def test_scitools():
    import scitools
    print "SciTools is installed in", os.path.dirname(scitools.__file__)
    print "SciTools version", scitools.__version__
    
    from scitools.configdata import config_parser_frontend
    config_data, files = config_parser_frontend('scitools', os.curdir)
    print "Values from scitools.cfg:"
    pprint.pprint(config_data)
    pprint.pprint(files)

def test_numpy():
    import numpy
    #numpy.test()

def test_numeric():
    import Numeric
    print "Numeric is installed in", os.path.dirname(Numeric.__file__)
    print "Numeric version", Numeric.__version__

def test_numarray():
    import numarray
    print "numarray is installed in", os.path.dirname(numarray.__file__)
    print "numarray version", numarray.__version__

def test_scipy():
    import scipy
    print "SciPy is installed in", os.path.dirname(scipy.__file__)
    print "SciPy version", scipy.version.version
    #scipy.test()

def test_blt():
    import Pmw
    import Tkinter

    print "Pmw is installed in", os.path.dirname(Pmw.Blt.__file__)
    print "Pmw version", Pmw.version()
    print "Tkinter is installed in", os.path.dirname(Tkinter.__file__)
    print "Tkinter version", Tkinter.__version__

    master = Tkinter.Tk()
    master.withdraw()

    root = Tkinter.Toplevel(master)
    root.title("Hello BLT!")
    root.geometry('320x240')
    
    def close_window(event=None):
        root.withdraw()
    root.protocol("WM_DELETE_WINDOW", close_window)
    root.bind("<KeyPress-q>", close_window)
    
    frame = Tkinter.Frame(root)
    frame.pack(side='top', fill='both', expand=1)

    g = Pmw.Blt.Graph(frame)
    g.pack(expand=1, fill='both')

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    g.line_create('x^2', xdata=tuple(x), ydata=tuple(y))

    g.update()

    wait()

def test_dx():
    import DX
    import Tkinter
    import tempfile

    DXMACROS = '/usr/share/dx/samples/macros'
    DXMACROS = os.environ.get('DXMACROS', DXMACROS)

    master = Tkinter.Tk()
    master.withdraw()
    root = Tkinter.Toplevel(master)
    root.title("Hello OpenDX!")
    def close_window(event=None):
        root.withdraw()
    root.protocol("WM_DELETE_WINDOW", close_window)
    root.bind("<KeyPress-q>", close_window)
    root.geometry('320x240')
    root.withdraw()
    frame = Tkinter.Frame(root, relief='sunken', bd=2)
    frame.pack(side='top', fill='both', expand=1)

    # start DX:
    cmd = DX.DXEXEC + ' -execonly -hwrender opengl'
    conn = DX.DXLStartDX(cmd, None)

    #DX.DXLLoadMacroDirectory(self.conn, DXMACROS)
    # hack for loading necessary macro files:
    DX.exDXLLoadScript(conn, os.path.join(DXMACROS, "ArrangeMemberMacro.net"))
    DX.exDXLLoadScript(conn, os.path.join(DXMACROS, "CappedIsoMacro.net"))


    root.deiconify()
    root.update()
    DX.DXLExecuteOnChange(conn)
    DX.WaitForDXIdle(conn)

def test_gnuplot():
    import Gnuplot

    gnuplot_command = Gnuplot.GnuplotOpts.gnuplot_command
    failure, out, err = get_status_output_errors(gnuplot_command + " --version")
    if failure:
        pass  # gnuplot command probably missing, will fail later

    print "Gnuplot.py is installed in", os.path.dirname(Gnuplot.__file__)
    print "Gnuplot.py version", Gnuplot.__version__
    print "Gnuplot version", out

    print "Values for some Gnuplot.py options:"
    for opt, value in Gnuplot.GnuplotOpts.__dict__.items():
        if opt == '__doc__':
            continue
        print "%s: %s" % (opt, value)
    print

    g = Gnuplot.Gnuplot(debug=1)
    g('set title "Hello Gnuplot!"')
    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    data = Gnuplot.Data(x, y, with_='lines')
    g.plot(data)
    wait()
    g.close()

def test_matplotlib():
    import matplotlib
    import matplotlib.colors

    # Override system defaults before importing pylab
    matplotlib.use('TkAgg')
    #matplotlib.rc('text', usetex=True)
    matplotlib.interactive(True)
    from matplotlib.font_manager import fontManager, FontProperties
    import pylab

    print "matplotlib is installed in", os.path.dirname(matplotlib.__file__)
    print "matplotlib version", matplotlib.__version__
    print "matplotlib.rcParams:"
    pprint.pprint(matplotlib.rcParams)

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    pylab.plot(x, y, 'bo-', linewidth=2.0)

    pylab.title("Hello Matplotlib!")

    pylab.draw()
    #pylab.show()  # requires manual quit of plot window
    time.sleep(0.5)

def test_mlabwrap():
    try:
        from scikits import mlabwrap
        from scikits.mlabwrap import mlab
    except ImportError, msg:
        print "Unable to find module scikits or scikits.mlabwrap"
        print "Error:\n", msg
        try:
            import mlabwrap
            from mlabwrap import mlab
        except ImportError, msg:
            # mlabwrap is not available
            print "Unable to find module mlabwrap"
            print "Error:\n", msg
            sys.exit(1)

    print "mlabwrap is installed in", os.path.dirname(mlabwrap.__file__)
    print "mlabwrap version", "unknown"
    print "MLABRAW_CMD_STR:", os.getenv("MLABRAW_CMD_STR")

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]

    mlab.plot(x, y)

    mlab.title("Hello mlabwrap!")

    wait()

def test_visit():
    import visit

    VISIT_ARGS = os.environ.get('VISIT_ARGS', ["-nosplash"])
    if isinstance(VISIT_ARGS, str):
        VISIT_ARGS = VISIT_ARGS.split()
    for arg in VISIT_ARGS:
        visit.AddArgument(arg)
    visit.Launch()

    print "visit is installed in", os.path.dirname(visit.__file__)
    print "visit version", visit.Version()

    visit.ResetView()
    v3D = visit.GetView3D()
    v3D.SetViewUp(0,0,1)
    v3D.SetViewNormal(-0.5,-0.8,0.4)
    v3D.SetImageZoom(1.0)
    visit.SetView3D(v3D)

    aa = visit.AnnotationAttributes()
    aa.SetAxesType(2)  # outside edges
    aa.SetDatabaseInfoFlag(False)
    aa.SetUserInfoFlag(False)
    visit.SetAnnotationAttributes(aa)

    t = visit.CreateAnnotationObject("Text2D")
    t.SetText("Hello VisIt!")
    t.SetPosition(0.4,0.9)  # (0,0) is lower left corner
    t.SetFontFamily(0)      # 0: Arial, 1: Courier, 2: Times
    t.SetWidth(0.25)        # 25%
    t.SetTextColor((0,0,0)) 
    t.SetUseForegroundForTextColor(False)
    t.SetVisible(True)

    visit.OpenDatabase("tmp_.vtk")

    visit.AddPlot("Mesh", "scalars")
    ma = visit.MeshAttributes()
    visit.SetPlotOptions(ma)
    visit.AddPlot("Pseudocolor", "scalars")
    pa = visit.PseudocolorAttributes()
    visit.SetPlotOptions(pa)

    visit.RedrawWindow()
    visit.DrawPlots()

    wait()

def test_vtk():
    import vtk
    import Tkinter
    try:
        from vtkTkRenderWidget import vktTkRenderWidget
    except:
        from vtk.tk.vtkTkRenderWidget import vtkTkRenderWidget

    print "vtk is installed in", os.path.dirname(vtk.__file__)
    print "vtk version", vtk.vtkVersion().GetVTKVersion()

    # Create a sphere source, mapper, and actor
    sphere = vtk.vtkSphereSource()

    sphereMapper = vtk.vtkPolyDataMapper()
    sphereMapper.SetInputConnection(sphere.GetOutputPort())
    sphereMapper.GlobalImmediateModeRenderingOn()
    sphereActor = vtk.vtkLODActor()
    sphereActor.SetMapper(sphereMapper)

    # Create a scaled text actor. 
    # Set the text, font, justification, and properties (bold, italics,
    # etc.).
    textActor = vtk.vtkTextActor()
    textActor.ScaledTextOn()
    textActor.SetDisplayPosition(90, 50)
    textActor.SetInput("Hello VTK!")

    # Set coordinates to match the old vtkScaledTextActor default value
    textActor.GetPosition2Coordinate().SetCoordinateSystemToNormalizedViewport()
    textActor.GetPosition2Coordinate().SetValue(0.6, 0.1)

    tprop = textActor.GetTextProperty()
    tprop.SetFontSize(24)
    tprop.SetFontFamilyToArial()
    tprop.SetJustificationToCentered()
    tprop.BoldOn()
    tprop.ItalicOn()
    tprop.ShadowOn()
    tprop.SetColor(0, 0, 1)

    # Create the Renderer, RenderWindow, RenderWindowInteractor
    master = Tkinter.Tk()
    master.withdraw()
    root = Tkinter.Toplevel(master)
    root.title("Hello VTK!")
    def close(event=None):
        root.withdraw()
    root.bind("<KeyPress-q>", close)
    frame = Tkinter.Frame(root, relief='sunken', bd=2)
    frame.pack(side='top', fill='both', expand=1)
    tkw = vtkTkRenderWidget(frame, width=320, height=240)
    tkw.pack(expand='true', fill='both')
    
    ren = vtk.vtkRenderer()
    renwin = tkw.GetRenderWindow()
    renwin.AddRenderer(ren)

    # Add the actors to the renderer; set the background and size; zoom
    # in; and render.
    ren.AddActor2D(textActor)
    ren.AddActor(sphereActor)

    ren.SetBackground(1, 1, 1)
    ren.ResetCamera()
    ren.GetActiveCamera().Zoom(1.5)

    root.update()
    renwin.Render()

    wait()

def test_pyx():
    import math
    import pyx

    print "PyX is installed in", os.path.dirname(pyx.__file__)
    print "PyX version", pyx.__version__

    g = pyx.canvas.canvas()
    g.items = []

    xpos = 0
    ypos = 0
    width = 15
    ratio = 0.5*(math.sqrt(5)+1)  # golden mean
    height = (1.0/ratio)*width
    
    graph = pyx.graph.graphxy(xpos, ypos, width=width, height=height)
    g.insert(graph)  # insert graph into figure canvas
    g1 = g.items[-1]

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    data = []
    for i in range(len(x)):
        data.append([x[i], y[i]])

    color = pyx.graph.style.color.cmyk.Blue
    styles = [pyx.graph.style.line([color])]
    g1.plot(pyx.graph.data.points(data, x=1, y=2), styles=styles)

    g1.text(g1.width/2+g1.xpos, g1.height+0.2+g1.ypos, "Hello PyX!",
            [pyx.text.halign.center,
             pyx.text.valign.bottom,
             pyx.text.size.Large])
    g.writetofile('tmp_pyx_.eps')

def test_grace(): 
    try:
        import grace_np
    except ImportError, msg:
        print "Unable to find module grace_np"
        print "Error:\n", msg
        try:
            import pygrace
            from pygrace import grace_np
        except ImportError:
            # grace_np is not available
            print "Unable to find module grace_np"
            print "Error:\n", msg
            sys.exit(1)

    g = grace_np.GraceProcess()
    g('arrange(1, 1, 0.1, 0.5, 0.6)')
    g('with g%s' % 0)
    g('g%s on' % 0)

    g('s0 on')
    g('s0 symbol 9')  # cross
    g('s0 symbol fill pattern 0')
    g('s0 symbol size 0.6')
    g('s0 symbol color 4')  # blue symbol
    g('s0 line color 2') # red line
    g('s0 linestyle 1')  # solid line

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    for i in range(len(x)):
        g('s0 point %s, %s' % (x[i], y[i]))
    g('autoscale')

    g('title "Hello Grace!"')
    g('redraw')
    
    wait()
    g('exit')

def test_veusz():
    import veusz
    import veusz.utils

    print "Veusz is installed in", os.path.dirname(veusz.__file__)
    print "Veusz version", veusz.utils.version()

    import veusz.embed
    g = veusz.embed.Embedded()
    g.EnableToolbar(enable=True)
    #g.window.hide()
    
    g.To('/')
    g.To(g.Add('page'))
    g.To('/page1')
    g.To(g.Add('graph'))

    xy = g.Add('xy')
    ## g.Set('%s/PlotLine/color' % xy, color)
    ## g.Set('%s/PlotLine/style' % xy, style)
    ## g.Set('%s/marker' % xy, marker)
    ## g.Set('%s/MarkerLine/color' % xy, color)
    ## g.Set('%s/MarkerFill/hide' % xy, True)

    x = [0, 1, 2, 3, 4]
    y = [0, 1, 4, 9, 16]
    g.SetData('x1', x)
    g.SetData('y1', y)
    g.Set('%s/xData' % xy, 'x1')
    g.Set('%s/yData' % xy, 'y1')

    label = g.Add('label')
    g.Set('%s/label' % label, "Hello Veusz!")
    g.Set('%s/xPos' % label, 0.2)
    g.Set('%s/yPos' % label, 0.9)

    g.Set('Background/hide', True)
    g.window.showNormal()

    wait(2)

def test_easyviz():
    from scitools.std import linspace, ndgrid, plot, contour, peaks, \
         quiver, surfc, backend, get_backend
    n = 21
    x = linspace(-3, 3, n)
    xx, yy = ndgrid(x, x, sparse=False)

    # a basic plot
    plot(x, x**2, 'bx:')
    wait()

    if backend in ['gnuplot', 'vtk', 'matlab', 'dx', 'visit', 'veusz']:
        # a contour plot
        contour(peaks(n), title="contour plot")
        wait()

        # a vector plot
        uu = yy
        vv = xx
        quiver(xx, yy, uu, vv)
        wait()

        # a surface plot with contours
        zz = peaks(xx, yy)
        surfc(xx, yy, zz, colorbar=True)
        wait()

    if backend == 'grace':
        g = get_backend()
        g('exit')

def test_dx():
    raise NotImplementedError('Tests for dx not implemented')

def test_latex():
    latexfile = 'tmp___latex'
    f = open(latexfile + '.tex', 'w')
    f.write(r"""
\documentclass{article}
\begin{document}
\[ \sin(x) = \sqrt{1 - \cos^2(x)} \]
\end{document}
""")
    f.close()
    status, outout, errout = get_status_output_errors('latex ' + latexfile)
    for filename in glob.glob(latexfile + '.*'):
        os.remove(filename)
    print 'status:', status, isinstance(status, int)
    if status != 0:
        raise Exception('status=%d, latex does not work: ' % status + outout)
          
def run():
    if not os.path.isfile('diagnostic.py'):
        print 'diagonstic.py must be run from the directory where it is located!'
        sys.exit(1)

    diag_fname = 'scitools_diagnostic.log'
    diag_file = open(diag_fname, 'w')

    #from scitools.misc import hardware_info
    #import pprint; pprint.pprint(hardware_info())
      
    # Write some info about the system to diag file:
    message("%s\n" % sys.version, diag_file, False)
    message("%s\n" % ' '.join(platform.uname()), diag_file, False)

    backends = ['blt', 'dx', 'gnuplot', 'grace', 'matplotlib',
                 'mlabwrap', 'pyx', 'veusz', 'visit', 'vtk']
    all_tests = ['scitools', 'numpy', 'scipy', 'latex'] + backends
                 
    for test in all_tests:
        message("\n" + "-"*30 + test + "-"*30 + "\n", diag_file, False)
        message("Running tests for %s..." % test, diag_file)
        cmd = 'python -c "import diagnostic; diagnostic.test_%s()"' % test
        failure, out, err = get_status_output_errors(cmd)
        if failure:
            message("failed\n", diag_file)
            message("Output:\n" + out, diag_file, False)
            message("Error:\n" + err, diag_file, False)
        else:
            message("ok\n", diag_file)
            message("Output:\n" + out, diag_file, False)
            if test in backends:
                message("Testing %s backend in Easyivz..." % test, diag_file)
                cmd = 'python -c "import diagnostic;' \
                      'diagnostic.test_easyviz()" ' \
                      '--SCITOOLS_easyviz_backend %s' % test
                failure, out, err = get_status_output_errors(cmd)
                if failure:
                    message("failed\n", diag_file)
                    message("Output:\n" + out, diag_file, False)
                    message("Error:\n" + err, diag_file, False)
                else:
                    message("ok\n", diag_file)
                    message("Output:\n" + out, diag_file, False)
        message("\n", diag_file, False)
            
    diag_file.close()
    message("See %s for details." % diag_fname)

if __name__ == '__main__':
    run()



#!/usr/bin/env python

"""
Regression module for automating regression tests.

    ================   =================================================
          Class                          Description
    ================   =================================================
    TestRun            utilities for easy set up of regression tests
    TestRunNumerics    extensions of TestRun for numerical computing
    Verify             search for test scripts, run them, compare
                       new results with reference results
    VerifyDiffpack     tailored Verify for Diffpack (incl. compilation)
    FloatDiff          visual diff between files with real numbers
    ================   =================================================

Usage
-----

Let us start with a very simple Python script, circle.py, for which we
would like to create a regression test::

  #!/usr/bin/env python
  '''Numerical integration of circular motion.'''
  import math, sys, os

  R=1; w=2*math.pi;  # global constants
  def advance(x, y, dt, t):
      '''advance (x,y) point one time step dt with Forward Euler,
         the equations describe circular motion of a body:
         dx/dt = -w*R*cos(2pi*w*t), dy/dt = w*R*sin(2pi*w*t)
      '''
      x = x - dt*w*R*math.sin(w*t);  y = y + dt*w*R*math.cos(w*t)
      return (x,y)

  # integrate from 0 to tstop
  try:
      tstop = float(sys.argv[1]); dt = float(sys.argv[2])
  except:
      print 'Usage: %s tstop dt' % sys.argv[0]; sys.exit(1)

  # make output format compatible with the plotpairs.py script:
  xmax = R*1.8; xmin = -xmax; ymin = xmin; ymax = xmax
  print xmin, xmax, ymin, ymax
  n = int(tstop/dt) + 1;
  x = R; y = 0
  for i in range(n):
      t = i*dt
      x, y = advance(x, y, dt, t)
      print x, y
  print 'end'

This script takes two input parameters from the command line:
the stop time for the simulation, and the time step.
The output is a series of numbers. We may run the script as::

  unix> python circle.py 3 0.1 > result

Let us assume that we are sure that the produced results in this
case are correct. A regression test then means that we can
automatically (in the future) run the above command and compare
the new results with archived correct results.

The procedure for using the Regression module in this examples
goes as follows.

  1. Create a name of the test, say "circle".
  2. Create circle.verify for running the test and saving
     desired results in a file circle.v.
  3. If we believe the results are correct, copy circle.v to
     circle.r. This circle.r file represents the archived correct
     results.

Later, we may run::

  unix> regression.py verify circle

to rerun the test and investigate differences between circle.v
(new results) and circle.r (archived correct results).
A whole directory tree can be examined for tests (.verify files)
by a similar command::

  unix> regression.py verify root-of-tree

Sometimes the new results are correct, but there are significant
differences between old and new results, because of, e.g., a change
in output formatting. New results can in this case be updated to
archive status by::

  unix> regression.py update root-of-tree

For the circle.py script, a typical circle.verify script takes
the following trivial form if we write it in bash::

  #!/bin/sh
  ./circle.py 3 0.1 > circle.v

The Regression module has tools for running programs, automatically
measuring CPU time, selecting lines from a file to put in the .v
file, ignoring round-off errors in numerical results, etc.
A more sophisticated test, which also accounts for numerical precision
in the output, goes as follows (to understand the various statements
you will need to consult Appendix B.4 in the textbook "Python Scripting
for Computational Science", by H. P. Langtangen)::

  #!/usr/bin/env python
  import os, sys
  from Regression import TestRunNumerics, defaultfilter
  test = TestRunNumerics('circle2.v')
  test.run('circle.py', options='1 0.21')
  # truncate numerical expressions in the output:
  test.approx(defaultfilter)
  # generate circle2.vd file in correct format:
  fd = open('circle2.vd', 'w')
  fd.write('## exact data\n')
  # grab the output from circle.py, throw away the
  # first and last line, and merge the numbers into
  # one column:
  cmd = 'circle.py 1 0.21'
  output = os.popen(cmd)
  res = output.readlines()
  output.close()
  numbers = []
  for line in res[1:-1]: # skip first and last line
      for r in line.split():
          numbers.append(r)
  # dump length of numbers and its contents:
  fd.write('%d\n' % len(numbers))
  for r in numbers: fd.write(r + '\n')
  fd.close()

This sample script can be adapted to a wide range of cases.
"""

import os, time, sys, string, re
from scitools.misc import system as os_system
from types import *
try:
    import Tkinter, Pmw
    _has_TkPmw = True
except ImportError:
    # cannot run floatdiff tools
    _has_TkPmw = False
    pass


class TestRun:
    """
    Utility for writing individual regression tests.
    Example on usage:
    from scitools.Regression import TestRun
    test = TestRun("test1.v")
    # run a program, place output on test1.v:
    test.run("myprog", options="-g -p 3.2", inputfile="test1.i")
    # append a file to test1.v:
    test.append("res1.dat")
    """


    def __init__(self,
                  logfile,         # .v file
                  removepath=' '   # remove this from sys.argv[0] output
                  ):
        """Clean up logfile, create a new one with header."""

        # use absolute path such that it is easy to write to the
        # logfile even after an os.chdir is performed

        self.logfile  = os.path.join(os.getcwd(),logfile)
        if string.find(logfile, '.v') == -1:
            raise ValueError('error in logfile name; must contain .v suffix')

        # remove logfile if it exists:
        if os.path.isfile(self.logfile): os.remove(self.logfile)

        # remove the detailed version as well:
        self.dlogfile = logfile + '-d'
        if os.path.isfile(self.dlogfile): os.remove(self.dlogfile)

        # make the logfile:
        vfile = open(self.logfile, 'w')
        tm = time.localtime(time.time())
        removepath = removepath + '/'
        self.scriptfilename = string.replace(sys.argv[0],removepath,'')
        vfile.write('%s: test performed on %d.%02d.%02d ' % \
                    (self.scriptfilename,tm[0],tm[1],tm[2]))
        if os.name == 'posix':  # is 'posix', 'nt' or 'mac'
            # unix
            u = os.uname()
            #vfile.write('(%s %s running %s)' % (u[1],u[4],u[0]))
            # this information is repeated at the end
        vfile.write('\n\n')
        vfile.close()

        # make a scratch directory for storing, e.g., gif-files:
        if 'HOME' in os.environ:
            self.scratchdir = os.path.join(os.environ['HOME'], 'tmp')
        else:
            if sys.platform.startswith('win'):
                self.scratchdir = r'C:\regression_temp'
            else:
                self.scratchdir = '/tmp/regresson_temp'
        if os.path.isfile(self.scratchdir):
            os.remove(self.scratchdir)
        if not os.path.isdir(self.scratchdir):
            os.mkdir(self.scratchdir)

        # add current working directory to the path:
        os.environ['PATH'] = os.environ['PATH'] + os.pathsep + os.curdir

    def run(self, program, options='', inputfile=''):
        """Run program, store output on logfile."""
        # the logfile is always opened in the constructor so
        # we can safely append here
        vfile = open(self.logfile, 'a')
        vfile.write('\n#### Test: %s' % (self.scriptfilename))
        vfile.write(' running %(program)s %(options)s\n' % vars())
        vfile.close()

        # do not use time.clock() to measure CPU time; it will not
        # notice the CPU time(here waiting time) of a system command
        t0 = os.times()  # [user,system,cuser,csystem,elapsed]
        if inputfile == '':
            cmd = '%s %s >> %s' % (program,options,self.logfile)
        else:
            cmd = '%s %s < %s >> %s' % (program,options,inputfile,self.logfile)
        failure, output = os_system(cmd, failure_handling='silent')
        if failure:
            vfile = open(self.logfile, 'a')
            msg = 'ERROR in %s: execution failure arose from the ' \
                  'command\n  %s\n\n%s\n\n' % (self.scriptfilename,cmd,output)
            vfile.write(msg)
            vfile.close()
            print msg
        # write CPU time of system command(user+system time
        # of child processes):
        t1 = os.times(); tm = t1[2] - t0[2] + t1[3] - t0[3]
        vfile = open(self.logfile, 'a')
        vfile.write('CPU time of %(program)s: %(tm).1f seconds' % vars())
        #if re.search('(x|sun)',sys.platform):
        if os.name == 'posix':  # is 'posix', 'nt' or 'mac'
            # unix
            u = os.uname()
            vfile.write(' on %s %s, %s' % (u[1],u[4],u[0]))
        vfile.write('\n\n')

    def loadfile(self, file):
        """Return a file as a list of lines for text processing."""

        if not os.path.isfile(file):
            print 'File',file,'does not exist'; sys.exit(1)
        FILE = open(file, 'r')
        lines = FILE.readlines()
        FILE.close()
        return lines

    def grepfile(self, regex, file, return_lineinfo=1):
        """
        Return a text consisting of the lines matching regex
        (regex can be string or list of strings, and
        return_lineinfo is true if each matched line is
        prefixed with the filename and the line file as
        a list of lines for text processing).
        """

        if type(regex) == type(''):
            regex = [regex]  # assume regex is a list of regex

        if not os.path.isfile(file):
            print 'File',file,'does not exist'; sys.exit(1)

        FILE = open(file, 'r')
        lines = FILE.readlines()
        FILE.close()
        matched_lines = []
        line_counter = 0
        for line in lines:
            line_counter += 1
            for r in regex:
                if re.search(r, line):
                    if return_lineinfo:
                        line = '%s %5d: ' % (file,line_counter) + line
                    matched_lines.append(line)
        return string.join(matched_lines, '\n')

    def write(self, text):
        """Write message to logfile."""
        vfile = open(self.logfile, 'a')
        vfile.write('\n--------------------------------------------------\n')
        vfile.write(text + '\n')
        vfile.write(  '--------------------------------------------------\n')
        vfile.close()

    def silentrun(self, program, options=''):
        """Run program without storing output on logfile."""

        if os.name == 'posix':
            # can write to /dev/null:
            failure = os.system('%s %s > /dev/null' % (program,options))
        else:
            failure = os.system('%s %s' % (program,options))
        if failure:
            vfile = open(self.logfile, 'a')
            msg = 'ERROR in %s: execution failure with %s %s\n' % \
                 (self.scriptfilename,program,options)
            vfile.write(msg)
            print msg

    def graphics(self, program, options=''):
        """
        Run e.g. graphics program if the environment variable
        BATCH_REGRESSION is not set.
        """
        batch = int(os.environ.get('BATCH_REGRESSION', 0))
        if not batch:
            failure, output = os_system('%s %s' % (program,options),
                                        failure_handling='silent')
            if failure:
                vfile = open(self.logfile, 'a')
                msg = 'ERROR in %s: execution failure with %s %s\n%s\n' % \
                      (self.scriptfilename,program,options,output)
                vfile.write(msg)
                print msg

    def append(self, file, maxlines=0):
        """Append a file or a list of files to the logfile."""

        vfile = open(self.logfile, 'a')

        # We need different code depending in whether file is a string
        # (a filename) or a list of filenames(e.g. from glob.glob).
        # The types module defines the names of the built-in types
        # in Python(here we need StringType and ListType)

        if isinstance(file, str):
            filelist = [file]  # make a list
        elif isinstance(file, list):
            filelist = file
        else:
            print 'ERROR in %s: append(file,...), the arg is of illegal '\
                  'type %s' % (self.scriptfilename,type(file))

        for f in filelist:
            if not os.path.isfile(f):
                vfile.write('%s: No file named %s was found by Regression.TestRun.append' % \
                            (self.scriptfilename,f))
                print 'No file named',f,'was found by Regression.TestRun.append'
                sys.exit(1)
            FILE = open(f, 'r')
            lines = FILE.readlines()
            FILE.close()
            vfile.write('\n\n----- appending file %s ' % f)
            if maxlines <= 0:
                printlines=len(lines)   # print all lines
            else:
                printlines = maxlines
                if printlines > len(lines):
                    printlines = len(lines)
                else:
                    vfile.write('(just %d lines) ' % printlines)
            vfile.write('------\n')
            #print 'treating file',f,'with',printlines,'lines'
            for i in range(printlines):
                vfile.write(lines[i])
        vfile.close()

    def picture(self, psfile):
        """Insert HTML commands for a gif picture."""
        # convert to gif:
        filestem = re.sub(r'(.*)\.[e]?ps', '', psfile)
        print 'picture: psfile=',psfile,'filestem=',filestem
        os_system('convert %s gif:%s.gif' % (psfile, filestem))
        pid = os.getpid()  # use pid to makea unique giffile name
        giffile_with_full_path = '%s/%s-%d.gif' % \
                                (self.scratchdir,pid,filestem)
        os.rename('%s.gif %s' % (filestem, giffile_with_full_path))
        self._insertgif (giffile_with_full_path)

    def movie(self, list_of_psfiles):
        """Insert HTML commands for an animated gif picture."""
        # convert to animated gif:
        filelist = string.join(list_of_psfiles, ' ')
        pid = os.getpid()  # use pid to make unique giffile name
        giffile_with_full_path = '%s/anim-%d.gif' % \
                                (self.scratchdir,pid)
        print 'making an animated gif sequence of psfiles:',filelist
        os_system('convert -loop 60 -delay 10 %s %s' % \
                  (filelist,giffile_with_full_path))
        self._insertgif (giffile_with_full_path)


    def _insertgif (self, giffile_with_full_path):
        vfile = open(self.logfile, 'a')
        vfile.write('<IMG SRC="%s">' % giffile_with_full_path)
        # always trigger a diff by inserting a comment containing time.time():
        vfile.write(' <!-- diff-triggering comment: %f -->\n<P>\n' %\
                     time.time())



class TestRunNumerics(TestRun):
    """
    Extends class TestRun with an additional logfile for
    dump of large sets of floating-point numbers for a second-level
    regression test.
    """

    def __init__(self, logfile, removepath='  '):
        TestRun.__init__(self, logfile, removepath)
        self.floatlogfile = logfile + 'd'
        if os.path.isfile(self.floatlogfile):
            os.remove(self.floatlogfile)
        # don't bring the float logfile into existence if not necessary


    def _approxline(self, line, realfilter):
        """
        Replace floating-point numbers in line by approximate
        numbers(computed by realfilter)(called from approx).
        """

        # the returned line has name newline
        newline = line

        # regex for real numbers (but not integers, \. must be a part!)
        c = re.compile(r"""
        (                            # start group
        [+\-]?\d\.\d+[Ee][+\-]\d\d   # -1.34E-01 etc
        |                            # OR
                                     # %f format with more than 3 decimals:
        [+\-]?\d+\.\d\d\d\d+         # -1.2556, not 7, not 7., not 7.22
        |                            # OR
        [+\-]?\.\d\d\d\d+            # .3656 (more than 4 decimals)
        )                            # end group
        """,
                       re.VERBOSE)
        # possible problem: 0 or 0.0 is never approximated by the
        # regex above (but 0.00E+00 is!)

        b = c.search(line)
        index = 0   # start of substring to test for next match
        while b:
            strno = b.group()
            no = float(strno)
            newno = realfilter(no)
            newline = string.replace(newline, strno, newno, 1)
            #print 'found %s, replaced by %s, to form\n   %s\n' % (strno, newno, newline)
            # b.end() refers to local index in current substring:
            index = index + b.end()
            # further search in the substring starting with index:
            b = c.search(line[index:])
        return newline

    def approx(self, realfilter):
        """
        Run through logfile, find all real numbers and replace
        them by approximate real numbers computed by realfilter.
        """

        logfile_copy = self.logfile+'.tmp'
        os.rename(self.logfile, logfile_copy)
        vfile = open(logfile_copy, 'r')
        afile = open(self.logfile, 'w') # new approximate logfile
        for line in vfile:
            # strip some digits:
            line = self._approxline(line, realfilter)
            # replace -0 by 0:
            line = re.sub(r'[eE]-00', 'e+00', line)
            line = re.sub(r'-0([, )])', '0\g<1>', line)
            afile.write(line)
        vfile.close()
        afile.close()

    def floatdump(self, program, options=''):
        """Run program and direct output to self.floatlogfile."""
        if not os.path.isfile(self.floatlogfile):
            # bring the float logfile into existence:
            ff = open(self.floatlogfile, 'w'); ff.close()

        cmd = '%s %s >> %s' % (program,options,self.floatlogfile)
        failure, output = os_system(cmd, failure_handling='silent')
        if failure:
            msg = 'ERROR in %s: execution failure arose from the ' \
                  'command\n  %s\n%s\n\n' % (self.scriptfilename,cmd,output)
            print msg

        # improvement: load output from system command into a list
        # of strings and examine the output format (must be in the
        # valid format!) OR: dump in a separate file, read it line
        # by line (better) and if the format is ok, append it

        # format specification: see test program at the end of the file

def defaultfilter(r):
    if abs(r) < 1.0E-14: r = 0.0  # set very small numbers to 0 exactly
    p = '%11.4e' % r
    return p

def exactfilter(r):
    if abs(r) < 1.0E-14: r = 0.0  # set very small numbers to 0 exactly
    p = '%g' % r                  # keep the precision
    return p

import stat
def walk(top, func, arg):
    """
    Simple copy of os.path.walk, but does not break when a file
    marked for visit is deleted during the process.
    """
    try:
        names = os.listdir(top)
    except os.error:
        return
    names.sort()
    func(arg, top, names)
    for name in names:
        name = os.path.join(top, name)
        try:
            st = os.lstat(name)
            if stat.S_ISDIR(st[stat.ST_MODE]):
                walk(name, func, arg)
        except:
            #print 'walk: could not work with the file',name
            #many files are naturally cleaned up during regression
            #tests (.o files e.g.)
            pass

class Verify:
    """
    Automates regression tests by running through a directory
    tree, searching for .verify files, executing them and
    comparing .v with .r files. The result of the comparison
    (the differing lines) are reported in HTML documents.
    """
    def __init__(self,
                 root='.',        # root directory or a single file
                 task='verify',   # 'verify' or 'update'
                 diffsummary = 'verify_log',
                 diffprog = None  # alternatives: diff, diff.py, diff.pl
                 ):
        # files are opened with changing os.getcwd so we need full path:
        self.diffsummary = os.path.join(os.getcwd(),
                                        diffsummary + '.htm')
        self.diffdetails = os.path.join(os.getcwd(),
                                        diffsummary + '_details.htm')
        if diffprog is None:
            self.diffprog = os.environ.get('DIFFPROG', 'diff.py')
            #self.diffprog = os.environ.get('DIFFPROG', 'diff')  # Unix diff
        else:
            self.diffprog = diffprog

        # remove old files, create new ones:
        if os.path.isfile(self.diffsummary):
            os.remove(self.diffsummary)
        if os.path.isfile(self.diffdetails):
            os.remove(self.diffdetails)

        # use font colors in the HTML file to indicated the kind of diff
        self.GREEN = 'green'; self.RED = 'red'

        # bring files into existence and write HTML header:
        ds = open(self.diffsummary, 'w');
        ds.write('<html><body>\n'\
                  '<font color=%s>%s color</font>: true differences<br>\n'\
                  '<font color=%s>%s color</font>: CPU time differences '\
                  'only\n<p>\n' % (self.RED,self.RED,self.GREEN,self.GREEN))
        ds.close()
        ds_d = open(self.diffdetails, 'w')
        ds_d.write('<html><body>\n')
        ds_d.close()
        self.testcounter = 0

        # the main action: run tests and diff!
        if os.path.isdir(root):
            # walk through a directory structure:
            walk(root, self._search4verify, task)
        elif os.path.isfile(root):
            # run just a single test:
            file = root   # root is just a file
            dirname = os.path.dirname(file)
            if dirname == '': dirname = os.getcwd()
            self._singlefile(dirname, task, os.path.basename(file))
        else:
            print 'Verify: root=',root,'does not exist'
            sys.exit(1)

        # write HTML footer:
        ds = open(self.diffsummary, 'a');
        ds.write('\n</body></html>\n')
        ds.close()
        ds_d = open(self.diffdetails, 'a')
        ds_d.write('\n</body></html>\n')
        ds_d.close()

    def _singlefile(self, dirname, task, file):
        """Run a single regression test."""
        # does the filename end with .verify?
        #if re.search(r'\.verify$', file):
        #    basename = re.sub(r'\.verify$', '', file)
        if file.endswith('.verify'):
            basename = file[:-7]
            if task == 'update':
                self._update(dirname, basename)
            elif task == 'verify':
                print '\n\nrunning verification test in', dirname,
                self._diff(dirname, basename, file)

    def _search4verify(self, task, dirname, files):
        """Called by function walk."""
        # change directory to current directory:
        origdir = os.getcwd(); os.chdir(dirname)
        for file in files:
            self._singlefile(dirname, task, file)
        if task == 'verify':
            self.clean(dirname)
        # change directory back again (required by walk):
        os.chdir(origdir)

    def _update(self, dirname, basename):
        vfile = basename + '.v';  rfile = basename + '.r'
        if os.path.isfile(vfile):
            os.rename(vfile, rfile)
            print '   %s -> %s in %s' % (vfile,rfile,dirname)
        vfile = basename + '.vd';  rfile = basename + '.rd'
        if os.path.isfile(vfile):
            os.rename(vfile, rfile)
            print '   %s -> %s' % (vfile,rfile)

    def _diff(self, dirname, basename, scriptfile):
        """Run script and find differences from reference results."""
        # run scriptfile, but ensure that it is executable
        os.chmod(scriptfile, 0755)
        # 0755: owner all, group+others can read and execute
        # 0644: owner r+w, group+others r
        self.run(scriptfile)

        # compare new output (.v) with reference results (.r)
        vfile = basename + '.v';  rfile = basename + '.r'
        if os.path.isfile(vfile):
            if not os.path.isfile(rfile):
                # if no rfile exists, copy vfile to rfile:
                os.rename(vfile, rfile)
            else:
                # compute difference:
                diffprog = self.diffprog
                if diffprog == 'diff.pl' or diffprog.startswith('diff.py'):
                    # diff.pl is slow for large files
                    # choose another program if filesize > 50K
                    Kb = os.path.getsize(vfile)/1000
                    if Kb > 50:
                        diffprog = 'diff -w'  # Unix C program
                        print 'Warning: switching diff program from',\
                              self.diffprog, 'to', diffprog
                diffcmd = '%s %s %s' % (diffprog,rfile,vfile)
                print '...' + diffcmd
                time.sleep(1)
                res = os.popen(diffcmd).readlines()
                ndifflines = len(res)
                summaryline = '%(dirname)s: %(rfile)s '\
                              '%(vfile)s   %(ndifflines)d lines' % vars()

                # no of tests so far:
                self.testcounter = self.testcounter + 1

                ds = open(self.diffsummary, 'a')
                if ndifflines == 0:
                    ds.write(summaryline); ds.write(' differ')
                else:
                    # write more detailed info about differences

                    # check if the diff lines are only CPU-time diffs
                    if self.diffCPUonly(res):
                        cpu_msg = ' (CPU time only!)'
                        font_color = self.GREEN
                    else:
                        cpu_msg = ''
                        font_color = self.RED
                    ds.write('<FONT COLOR=%s>%s</FONT>' % \
                             (font_color,summaryline))

                    # link to file with details:
                    anchor = 'part' + str(self.testcounter)
                    ds.write(' <A HREF="%s#%s">differ</A>%s' % \
                             (self.diffdetails,anchor,cpu_msg))

                    # write the detailed summary:
                    ds_d = open(self.diffdetails, 'a')
                    ds_d.write('<P><A NAME="%s">'\
                    '<B>%s differ</B></A>'\
                    '<BR>The differences were calculated by %s.'\
                    '<PRE>\n' % (anchor,summaryline,diffprog))
                    ds_d.writelines(res)
                    ds_d.write('\n</PRE>\n')
                    if diffprog != self.diffprog:
                        ds_d.write('<P><B>NOTE: %s was used instead of %s '\
                                   'because of the large filesizes!</B><P>' % \
                                   (diffprog,self.diffprog))

                    # check if floating-point files are present:
                    vfile = basename + '.vd';  rfile = basename + '.rd'
                    if os.path.isfile(vfile):
                        if not os.path.isfile(rfile):
                            os.rename(vfile, rfile)
                        else:
                            # write a shell file that can be
                            # executed from a link:
                            shfilename = os.path.join(os.getcwd(),'tmp.'+basename+'_fdiff.sh')
                            shfile = open(shfilename, 'w')
                            shfile.write('floatdiff.py %s %s\n' %\
                                         (vfile, rfile))
                            shfile.close()
                            os.chmod(shfilename, 0755)
                            ds_d.write(
                             '<P><A HREF="%s">Floating-point difference '\
                             'between %s and %s without any approximations</A>\n' % \
                               (shfilename,vfile, rfile))
                            # check if there _are_ differences...
                            if os.name == 'posix':
                                diff = os.popen('diff %s %s | wc -l' % (vfile,rfile)).readlines()
                                diff = int(diff[0])
                                if diff == 0:
                                    ds_d.write(' (no differences!)\n')

                    ds_d.close()

                ds.write('<BR>\n')
                ds.close()
        else:
            print 'ran %s, but no %s.v file found - check that %s.verify defines %s.v as logfile' % (scriptfile,basename,basename,basename)
            sys.exit(1)

    def diffCPUonly(self, difflines):
        # do all differing lines contain something with cpu or date?
        cpu_only = 1
        for line in difflines:
            # avoid special diff.pl lines and date:
            if  not re.search(r'---', line) \
            and not re.search(r'Diffpack Version ', line) \
            and not re.search(r': test performed on ', line) \
            and not re.search(r'^\d+c\d+$', line) \
            and not re.search(r'(Mon|Tue|Wed|Thu|Fri|Sat|Sun) \w{3}\s+\d+ \d\d:\d\d:\d\d \d{4}', line) \
            and not re.search(r'diff-triggering comment', line):
                if not re.search(r'cpu', line, re.IGNORECASE):
                    cpu_only = 0; break
        return cpu_only

    # the run function can be overrided in subclasses and tailored to special
    # tests where it is necessary to, e.g., compile special applications
    # prior to running the script

    def run(self, scriptfile):
        # recall that os.chdir has been taken to the scriptfiles's dir
        path = os.path.join(os.curdir, scriptfile)
        # path is executable since we made an os.chmod in self._diff
        failure, output = os_system(path, failure_handling='silent')
        if failure:
            print 'Failure in regression test', path
            print output

    def clean(self, dirname):
        return


class VerifyDiffpack(Verify):
    """
    Extend class Verify with compilation of Diffpack applications,
    clean-up of applications, and a parameter self.makemode for running
    applications in opt or nopt mode.
    """

    def __init__(self, root='.', task='verify',
                 diffsummary = 'verify_log',
                 diffprog = None,
                 makemode = 'opt'):
        self.makemode = makemode
        # run all the stuff:
        Verify.__init__(self, root, task, diffsummary, diffprog)

    def run(self, scriptfile):
        """Run script, but compile the application first."""
        # is this a Verify directory? (recall that we have chdir'ed to dir)
        if string.count(os.getcwd(), '/Verify'):
            # go to parent directory (os.pardir is '..'):
            thisdir = os.getcwd();  os.chdir(os.pardir)
            #print '\na Verify dir with a parent dir\n  ',os.getcwd(),\
            #      '\nLet's compile!\n'
            print '\n...compile app in', os.getcwd()
            if os.path.isfile('Makefile'):
                # yes, we have a makefile!
                failure, output = os_system('Make MODE=%s' % self.makemode,
                                            failure_handling='silent')
                if failure:
                    print 'Could not compile in directory', os.getcwd()
            os.chdir(thisdir) # back to Verify dir

            print '\n\n...run regression test ''+scriptfile+'' for VerifyDiffpack.run:'
        # call parent class' run function:
        Verify.run(self,scriptfile)

    def clean(self, dirname):
        """Clean up files, typically executables etc."""
        # is this a Verify directory? (recall that we have chdir'ed to dir)
        if string.count(os.getcwd(), '/Verify'):
            # go to parent directory and clean application:
            thisdir = os.getcwd();  os.chdir(os.pardir)
            if os.path.isfile('Makefile'):
                # yes, we have a makefile!
                failure, output = os_system('Make clean',
                                            failure_handling='silent')
                if failure:
                    print 'Could not run Make clean in directory', os.getcwd()
            os.chdir(thisdir)

class FloatDiff:
    def __init__(self, master, file1, file2):
        if not _has_TkPmw:  # global variable set in top of the script
            raise ImportError('Could not import Tkinter and Pmw')

        self.master = master
        self.top = Tkinter.Frame(master, borderwidth=5)
        self.top.pack()
        self.GUI = 1  # true or false
        list = self.loadfiles(file1, file2)
        self.buildGUI(list, file1, file2)

        if self.GUI:
            self.canvas.resizescrollregion() # make the scrollbars right

        return

    def loadfiles(self, file1, file2):
        """
        Purpose: build "list", a list of relevant info for
        all differences found between file1 and file2
        (list can afterwards be visualized in buildGUI).
        """

        # first check if the files are identical:
        if os.name == 'posix':
            diff = os.popen('diff %s %s | wc -l' % (file1,file2)).readlines()
            diff = int(diff[0])
            if diff == 0:
                print 'files %s and %s are identical, no need to launch GUI' \
                      % (file1,file2)
                self.GUI = 0
                return None

        list = []
        f1 = open(file1, 'r');  f2 = open(file2, 'r')
        while 1:
            line1 = f1.readline(); line2 = f2.readline()
            if not line1 or not line2: break
            if re.search('^##',line1):
                if line1 != line2:
                    # strange; the two comment lines are different
                    print 'two comment lines are different!'
                    print 'line1:\n  ', line1
                    print 'line2:\n  ', line2
                comment = line1
            line1 = f1.readline(); line2 = f2.readline()
            if not line1 or not line2:
                print 'wrong datafile format; '\
                      'comment line not proceeded by data'
                sys.exit(1)
            if line1 != line2:
                print '%s has %s items, whereas %s has %s items' % \
                     (file1,line1,file2,line2)
                break
            nitems1 = int(line1)
            differences = []
            max = -1E+20; min = -max; avg = 0;
            for i in range(nitems1):
                s1 = f1.readline(); s2 = f2.readline()
                if not s1 or not s2:
                    print 'wrong datafile format'; sys.exit(1)
                # test that only one number is there... (no space)
                if re.search(' ', s1) or re.search(' ', s2):
                    print 'wrong datafile format'
                    print 'lines:','\n',s1,'\n',s2; sys.exit(1)
                r1 = float(s1); r2 = float(s2)
                # statistics:
                if (r1 > max):  max = r1
                if (r1 < min):  min = r1
                avg = avg + r1
                # difference?
                if s1 != s2:
                    # report this difference:
                    r = r1 - r2
                    differences.append((i, s1, s2, str(r)))
            avg = avg / nitems1
            list.append((comment, differences, max, min, avg))
        f1.close(); f2.close()
        return list

    def buildGUI(self, list, file1, file2):
        """Build list of fields and canvas (text and plot)."""

        if not list or not self.GUI:
            return

        buttonframe = Tkinter.Frame(self.top)
        buttonframe.pack(side='left')
        canvasframe = Tkinter.Frame(self.top)
        canvasframe.pack(side='right')

        self.fieldlist = Pmw.ScrolledListBox(buttonframe,
                listbox_selectmode = 'extended',
                vscrollmode='dynamic',
                label_text = 'list of fields',
                labelpos = 'n',  # needed if label_text is present
                listbox_width = 20,
                listbox_height = 20,
                selectioncommand = self.selectfield)
        self.fieldlist.pack()

        self.canvas = Pmw.ScrolledCanvas(canvasframe,
            borderframe = 1, labelpos = 'n',
            label_text = 'intelligent float diff between %s and %s' \
                                          % (file1,file2),
            usehullsize = 1, hull_width = 800, hull_height = 500,
            vscrollmode = 'static', hscrollmode = 'static'
            )
        self.canvas.pack(fill='both', expand=1)

        counter = 0
        self.graphs = []
        self.textdiffs = []
        self.rdata = []
        self.vdata = []
        self.ycoor = []
        maxdiffs = 0
        for (comment, differences, max, min, avg) in list:
            n = len(differences)
            if n > maxdiffs: maxdiffs = n

        for (comment, differences, max, min, avg) in list:
            tag = 'tag' + str(counter)
            n = len(differences)
            if n == 0:
                continue  # no diff, proceed with next

            textdiff = Tkinter.Text(self.canvas.interior(),
                                    width=52,height=maxdiffs+3,wrap='none')
                                    #width=52,height=str(maxdiffs)+'c',wrap='none')
            self.textdiffs.append(textdiff)
            textwin = self.canvas.create_window(0,0, anchor='nw',
                      window=self.textdiffs[counter], tag=tag)
            textdiff.insert('end',
            'line: %-10s      %-10s      float diff\n\n' % (file1,file2))
            for (i,s1,s2,r) in differences:
                textdiff.insert('end', '%4d: ' % i)
                self.highlight(textdiff, s1.strip(), s2.strip())
                textdiff.insert('end', '%-16g\n' % float(r))

            # create graph:

            self.rdata.append(Pmw.Blt.Vector())
            self.vdata.append(Pmw.Blt.Vector())
            self.ycoor.append(Pmw.Blt.Vector())
            # note: these vectors are recreated in the next part
            # of the loop, must have a list of such vectors, indexed
            # by counter!
            for j in range(n): self.ycoor[counter].append(j)
            for (i,s1,s2,r) in differences:
                self.vdata[counter].append(float(s1))
                self.rdata[counter].append(float(s2))
                graph = Pmw.Blt.Graph(self.canvas.interior(), width=300,
                                      height=str(maxdiffs)+'c')
            self.graphs.append(graph)
            graph.line_create('v',
                              xdata=self.vdata[counter],
                              ydata=self.ycoor[counter],
                              label='new results',
                              color='green', linewidth=2,
                              dashes='', symbol='')
            graph.line_create('r',
                              xdata=self.rdata[counter],
                              ydata=self.ycoor[counter],
                              label='reference',
                              color='red', linewidth=2,
                              dashes='', symbol='')
            graph.configure(title='plot of %s vs %s' % (file1,file2))
            #self.vavg = Pmw.Blt.Vector(); self.vavg = [avg]*n # does not work!

            plotwin = self.canvas.create_window(420,0, anchor='nw',
                      window=self.graphs[counter], tag=tag)
            graph.xaxis_configure(min=min, max=max)
            if n > 1: ymax = n-1
            else:     ymax = 1
            graph.yaxis_configure(min=0, max=ymax, descending=1)

            # insert item in list:
            self.fieldlist.insert('end',comment.strip()) # strip trailing \n
#            button = Tkinter.Button(buttonframe,
#                                    text=comment,
#                     command=lambda f=self.lift, c=self.canvas,
#                     p=self.graphs[counter], t=self.textdiffs[counter]:
#                               f(c,p,t))
#            button.pack(side='top')
            counter = counter + 1
        Tkinter.Button(buttonframe, text='QUIT',
                       command=self.master.quit).pack(side='top',pady=5)

    def selectfield(self):
        counter = int(self.fieldlist.curselection()[0])
        self.lift(self.graphs[counter],self.textdiffs[counter])

    def lift(self, win1, win2):
        win1.lift()
        win2.lift()
        # adapt the scroll region:
        self.canvas.resizescrollregion()

        #does not work with embedded windows:
        #self.canvas.lift(win1)
        #self.canvas.component('canvas').lift(win1)

    def highlight(self, text, s1, s2):
        markers = [1] * max(len(s1),len(s2))
        for i in range(min(len(s1),len(s2))):
            if s1[i] == s2[i]: markers[i] = 0
        for i in range(len(s1)):
            if markers[i] == 0:
                text.insert('end', str(s1[i]), 'equal')
            else:
                text.insert('end', str(s1[i]), 'diff')
        for i in range(16-len(s1)):
            text.insert('end', ' ', 'equal')
        for i in range(len(s2)):
            if markers[i] == 0:
                text.insert('end', str(s2[i]), 'equal')
            else:
                text.insert('end', str(s2[i]), 'diff')
        text.tag_configure('diff', background='cyan')
        for i in range(16-len(s2)):
            text.insert('end', ' ', 'equal')


def verify_file_template(casename, floats=False):
    s = """\
#!/usr/bin/env python
import os, sys
from scitools.Regression import TestRunNumerics, defaultfilter
test = TestRunNumerics('%s.v')
test.run('%s.py', options='...')
# test.append(somefile)
# truncate numerical expressions in the output:
test.approx(defaultfilter)
""" % (casename, casename)
    if floats:
        s += """
# generate %s.vd file in correct format:
fd = open('%s.vd', 'w')
fd.write('## exact data\n')
# write out float data in some way
# ...
fd.close()
""" % (casename, casename)
    vf = casename + '.verify'
    f = open(vf, 'w')
    f.write(s)
    f.close()
    print 'a template regression script is written to', vf


# tests of the current module:

def _test_floatdiff():
    root = Tkinter.Tk()
    #Pmw.initialise(root, fontScheme='pmw1')
    root.title('intelligent float diff')

    # make two files with almost equal data:
    file1 = 'tmptest.v';  file2 = 'tmptest.r'
    f1 = open(file1, 'w');  f2 = open(file2, 'w')
    f1.write("""## field1
10
1.345
3.45
6.9
4
9
8.999999
1.065432E-01
1.07E-01
1.06E-01
0.04E-01
## field2
11
1.6
3.1
2.0
1.1
1.7
1.8
1.9
0
-9.3
-14.4
-1.6
## field3
10
1.9
3.3
2.7
0.8
0.88
0.8000
0.8001
0.8002
0.8003
0.8004
""")
    f2.write("""## field1
10
1.344
3.66
7.0
4
9
8.999999
1.065432E-01
1.07E-01
1.06E-01
1.0376243E-01
## field2
11
1.61
3.11
2.01
1.11
1.7
1.8
1.9
0
-9.3
-14.400
-1.6000E+00
## field3
10
1.9
3.3
2.7
0.8001
0.88001
0.8000001
0.8001
0.8002
0.8003
0.8001
""")
    f1.close(); f2.close()
    fd = FloatDiff(root, file1, file2)
    root.mainloop()

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print 'Usage: %s [template | verify | update | floatdiff] [verify/update-root]' % sys.argv[0]
        sys.exit(1)
    task = sys.argv[1]
    try:
        verifyroot = sys.argv[2]
    except:
        verifyroot = os.getcwd()

    if task == 'floatdiff':
        _test_floatdiff()
    elif task == 'template':
        verify_file_template('somecase')
    else:
        # task == 'verify' or task == 'update'
        v = Verify(verifyroot, task, 'verify_log')
        if task == 'verify':
            print 'check verify_log.htm'




#!/usr/bin/env python
"""
Module for managing parameters.
"""
import re, os, sys

def message(m):
    if os.environ.get('DEBUG', '0') == '1':
        print m
    

class PrmDictBase(object):
    """
Base class for managing parameters stored in dictionaries.
Typical use includes data or solver classes for solving physical
problems numerically. One may then choose to hold all physical
parameters in a dictionary physical_prm, containing
(parameter name, value) pairs, and all numerical parameters in
a dictionary numerical_prm. The physical_prm and numerical_prm
dictionaries can then be defined in a subclass of PrmDictBase
and managed by PrmDictBase. The management includes several
convenient features:

 - keeping all input data in one place
 - setting of one or more parameters where the type of the value
   must match the type of the previous (initial) value
 - pretty print of all defined parameters
 - copying parameters from dictionaries to, e.g., local variables
   and back again, or to local namespaces and back again
 - easy transition from parameter dictionaries to more sophisticated
   handling of input data, e.g., class scitools.ParameterInterface
   (GUI, CGI, command-line args)

The subclass typically defines the dictionaries, say
self.physical_prm and self.numerical_prm. Then these are
appended to the inherited self._prm_list list to be registered.
All members of this list are dictionaries that will not accept
new keys (i.e., all parameters must be defined prior to registering
them in self._prm_list). With this list one has a collection of all
parameter dictionaries in the application.

self._type_check[prm] is defined if we want to type check
a parameter prm.
if self._type_check[prm] is True (or False), prm must either
be None, of the same type as the previously registered
value of prm, or any number (float, int, complex) if
the previous value prm was any number. Instead of a boolean
value, self._type_check[prm] may hold a tuple of class types
(to be used in isinstance checks), or a function which takes
the value as argument and returns True if the that value is
of the right type (otherwise False).


In addition to the parameter dictionaries with fixed keys, class
PrmDictBase also holds a self.user_prm, which is a dictionary
of "meta data", i.e., an arbitrary set of keys and values that
can arbitrarily extended anywhere. If self.user_prm is None,
no such meta data can exists (implying that only parameters
registered in the dictionaries in self._prm_list are allowed - the
programmer of subclasses can of course extend these parameter
sets whenever desired; disallowing a parameter name is only a
feature of the set function for setting the value of a (registered)
parameter).

Here is an example::

    from scitools.PrmDictBase import PrmDictBase

    class SomeSolver(PrmDictBase):
        def __init__(self, **kwargs):
            PrmDictBase.__init__(self)
            # register parameters in dictionaries:
            self.physical_prm = {'density': 1.0, 'Cp': 1.0,
                                       'k': 1.0, 'L': 1.0}
            self.numerical_prm =  {'n': 10, 'dt': 0.1, 'tstop': 3}

            # attach dictionaries to base class list (required):
            self._prm_list = [self.physical_prm, self.numerical_prm]

            # specify parameters to be type checked when set:
            self._type_check.update({'n': True, 'dt': (float,),
                  'k': lambda k: isinstance(int,float) and k>0})

            # disallow arbitrary meta data
            self.user_prm = None # set to {} if meta data are allowed

            # initialize parameters according to keyword arguments:
            self.set(**kwargs)


        def _update(self):
            # dt depends on n, L, k; update dt in case the three
            # others parameters have been changed
            # (in general this method is used to check consistency
            # between parameters and perform updates if necessary)
            n = self.numerical_prm['n']
            L = self.physical_prm['L']
            k = self.physical_prm['k']

            self.u = zeros(n+1, Float)
            h = L/float(n)
            dt_limit = h**2/(2*k)
            if self.numerical_prm['dt'] > dt_limit:
                self.numerical_prm['dt'] = dt_limit

        def compute1(self):
            # compute something
            return self.physical_prm['k']/self.physical_prm['Cp']

        def compute2(self):
            # turn numerical parameters into local variables:
            exec self.dicts2variables(self._prm_list)
            # or exec self.dicts2variables(self.numerical_prm)  # selected prms

            # now we have local variables n, dt, tstop, density, Cp, k, L
            # that we can compute with, say

            Q = k/Cp
            dt = 0.9*dt

            # if some of the local variables are changed, say dt, they must
            # be inserted back into the parameter dictionaries:
            self.variables2dicts(self.numerical_prm, dt=dt)    

    """
    
    def __init__(self):
        # dicts whose keys are fixed (non-extensible):
        self._prm_list = []     # fill in subclass
        self.user_prm = None    # user's meta data
        self._type_check = {}   # fill in subclass

    def _prm_dict_names(self):
        """Return the name of all self.*_prm dictionaries."""
        return [attr for attr in self.__dict__ if \
                re.search(r'^[^_].*_prm$', attr)]
                
    def usage(self):
        """Print the name of all parameters that can be set."""
        prm_dict_names = self._prm_dict_names()
        prm_names = []
        for name in prm_dict_names:
            d = self.__dict__[name]
            if isinstance(d, dict):
                k = d.keys()
                k.sort(lambda a,b: cmp(a.lower(),b.lower()))
                prm_names += k
        print 'registered parameters:\n'
        for i in prm_names:
            print i
        # alternative (sort all in one bunch):
        # names = []
        # for d in self._prm_list:
        #     names += d.keys()
        # names.sort
        # print names

    def dump(self):
        """Dump all parameters and their values."""
        for d in self._prm_list:
            keys = d.keys()
            keys.sort(lambda a,b: cmp(a.lower(),b.lower()))
            for prm in keys:
                print '%s = %s' % (prm, d[prm])

    def set(self, **kwargs):
        """Set kwargs data in parameter dictionaries."""
        # print usage message if no arguments:
        if len(kwargs) == 0:
            self.usage()
            return
        
        for prm in kwargs:
            _set = False
            for d in self._prm_list:
                if len(d.keys()) == 0:
                    raise ValueError('self._prm_list is wrong (empty)')
                try:
                    if self.set_in_dict(prm, kwargs[prm], d):
                        _set = True
                        break
                except TypeError, msg:
                    print msg
                    #break
                    sys.exit(1)  # type error is fatal

            if not _set:   # maybe set prm as meta data?
                if isinstance(self.user_prm, dict):
                    # not a registered parameter:
                    self.user_prm[prm] = kwargs[prm]
                    message('%s=%s assigned in self.user_prm' % \
                            (prm, kwargs[prm]))
                else:
                    raise NameError('parameter "%s" not registered' % prm)
        self._update()

    def set_in_dict(self, prm, value, d):
        """
        Set d[prm]=value, but check if prm is registered in class
        dictionaries, if the type is acceptable, etc.
        """
        can_set = False
        # check that prm is a registered key
        if prm in d:
            if prm in self._type_check:
                # prm should be type-checked
                if isinstance(self._type_check[prm], (int,float)):
                    # (bool is subclass of int)
                    if self._type_check[prm]:
                        # type check against prev. value or None:
                        if isinstance(value, (type(d[prm]), None)):
                            can_set = True
                        # allow mixing int, float, complex:
                        elif operator.isNumberType(value) and\
                                 operator.isNumberType(d[prm]):
                            can_set = True
                elif isinstance(self._type_check[prm], (tuple,list,type)):
                    # self._type_check[prm] holds either the type or
                    # a tuple/list of types; test against them
                    #print 'testing %s=%s against type %s' % (prm,value,self._type_check[prm])
                    if isinstance(value, self._type_check[prm]):
                        can_set = True
                    else:
                        raise TypeError('\n\n%s=%s: %s has type %s, not %s' % \
                                        (prm, value, prm, self._type_check[prm],
                                         type(value)))

                elif callable(self._type_check[prm]):
                    can_set = self._type_check[prm](value)
                else:
                    raise TypeError('self._type_check["%s"] has an '\
                                    'illegal value %s' % \
                                    (prm, self._type_check[prm]))
            else:
                can_set = True
        else:
            message('%s is not registered in\n%s' % (prm, d))
        if can_set:
            d[prm] = value
            message('%s=%s is assigned' % (prm, value))
            return True
        return False
        

    def _update(self):
        """Check data consistency and make updates."""
        # to be implemented in subclasses
        pass

    def get(self, **kwargs):
        return [self._solver_prm[prm] \
                for prm in kwargs if prm in self._solver_prm]
                
    def properties(self, global_namespace):
        """Make properties out of local dictionaries."""
        for ds in self._prm_dict_names():
            d = eval('self.' + ds)
            for prm in d: # or for prm in self.__dict__[ds]
                # properties cannot have whitespace:
                prm = prm.replace(' ', '_')
                cmd = '%s.%s = property(fget='\
                      'lambda self: self.%s["%s"], %s)' % \
                      (self.__class__.__name__, prm, ds, prm,
                       ' doc="read-only property"')
                print cmd
                exec cmd in global_namespace, locals()

    def dicts2namespace(self, namespace, dicts, overwrite=True):
        """
        Make namespace variables out of dict items.
        That is, for all dicts, insert all (key,value) pairs in
        the namespace dict.
        namespace is a dictionary, dicts is a list of dictionaries.
        """
        # can be tuned in subclasses

        # allow dicts to be a single dictionary:
        if not isinstance(dicts, (list,tuple)):
            dicts = [dicts]

        for d in dicts:
            if overwrite:
                namespace.update(d)
            else:
                for key in d:
                    if key in namespace and not overwrite:
                        print 'cannot overwrite %s' % key
                    else:
                        namespace[key] = d[key]

    def dicts2namespace2(self, namespace, dicts):
        """As dicts2namespace2, but use exec."""
        # can be tuned in subclasses

        # allow dicts to be a single dictionary:
        if not isinstance(dicts, (list,tuple)):
            dicts = [dicts]
            
        for d in dicts:
            for key in d:
                exec '%s=%s' % (key,repr(d[key])) in globals(), namespace

    def namespace2dicts(self, namespace, dicts):
        """
        Update dicts from variables in a namespace.
        That is, for all keys in namespace, insert (key,value) pair
        in the dict in dicts that has the same key registered.
        namespace is a dictionary, dicts is a list of dictionaries.
        """
        # allow dicts to be a single dictionary:
        if not isinstance(dicts, (list,tuple)):
            dicts = [dicts]

        keys = []    # all keys in namespace that are keys in dicts
        for key in namespace:
            for d in dicts:
                if key in d:
                    d[key] = namespace[key]  # update value
                    keys.append(key)         # mark for delete
        # clean up what we made in self.dicts2namespace:
        for key in keys:
            del namespace[key]

    def dicts2variables(self, dicts):
        """
        Make Python code string that defines local variables from
        all parameters in dicts (list of dictionaries of parameters).
        For example, if dicts[1] has a key n with value 1.0, the
        statement 'n=1.0' will be included in the returned string.
        The calling code will typically exec this returned string
        to make local variables (short hands) from parameters stored
        in dictionaries. (Note that such local variables are read-only,
        changing their values will not be reflected in the dictionaries!).
        """
        # allow dicts to be a single dictionary:
        if not isinstance(dicts, (list,tuple)):
            dicts = [dicts]

        s = ''
        for d in dicts:
            for name in d:
                s += '%s = %s\n' % (name, d[name])
        return s

    def variables2dicts(self, dicts, **variables):
        """
        Insert the name=value keyword arguments in variables into
        the dictionaries in dicts (list of dictionaries).
        This is the inverse of the dicts2variables function.

        Usage:
        exec self.dicts2variables(self.numerical_prm)
        # work with read-only n, dt, tstop
        ...
        # update (in case n, dt, tstop was changed):
        self.variables2dicts(self.numerical_prm, n=n, dt=dt, tstop=tstop)
        """
        for name in variables:
            for d in dicts:
                if name in d:
                    d[name] = variables[name]
    

# initial tests are found in src/py/examples/classdicts.py



"""
Import a set of modules. If modules are missing,
one can get these as a comma-separated string, or
one can raise exceptions.

See test block for demo of how this module can be used.
"""

def missing(*module_names):
    """
    Check if modules can be imported.
    Return a string containing the missing modules.
    """
    missing_modules = []
    for module in module_names:
        try:
            exec("import " + module)
        except:
            missing_modules.append(module)
    s = ", ".join(missing_modules)
    return s


def exception(msg, *module_names):
    """
    Raise an ImportError exception if modules
    are missing. Print the names of the missing modules.
    """
    s = missing(*module_names)
    if s:
        raise ImportError("%s missing the module(s) %s" % (msg,s))

def message(module, critical=1, msg=None):
    """
    Import a module and write a message if it is missing.
    critical=0 means that the module is not critical
    (programs may work without). critical=1 means that the
    module must be installed.
    msg is an optional description of the module.
    """
    try:
        exec("import "+ module)
        print "You have the Python", module, "module installed..."
        success = True
    except:
        print "*** The", module, "Python module is not available..."
        success = False
        if msg: print "    (%s)" % msg
        if not critical:
            print "       ....but this is not critical"
            success = False
    return success

# Pmw.Blt.Graph requires a special treatment:
def PmwBlt(verbose=0):
    try:
        # check if we manage to create a Blt graph:
        import Pmw;  s = Pmw.Blt.Graph
        if verbose:
            print "You have the Python Pmw.Blt.Graph widget available..."
        return True
    except:
        if verbose:
            print "*** Python is not correctly linked with BLT (no Pmw.Blt.Graph widget)"
        return False


if __name__ == '__main__':
    # Suppose we have a class SomeClass requiring the modules
    # A, B, and C. The following code segment shows how you can
    # import SomeClass silently if one or more of the modules A, B, or
    # C are missing, and then get the error message of a missing
    # module when you create an instance of the class

    try:
        import A, B, C
    except:
        pass

    class SomeClass:
        def __init__(self, *args):
            import modulecheck
            modulecheck.exception("SomeClass:", "A", "B", "C")

            # if we come here, no exception from modulecheck.exception
            # was raised so we can continue with the constructor statements,
            # e.g.
            a = 1
            b = 2

    # this should raise an exeption! (as the modules A, B, and C are missing)
    try:
        a = SomeClass()
    except:
        import sys
        print "Test worked; ", "\n", sys.exc_type, "\n", sys.exc_value

    # this should work fine:
    if exception("just testing...", "os"):  print "bug!"



from scitools.std import *
n = 20  # no of periods of a sine function
r = 80  # resolution of each period
x = linspace(0, n, r*n + 1)
amplitude = 1 + sin(2*pi*0.05*x)
y = amplitude*sin(2*pi*x)

r = (y.max() - y.min())/x[-1]
print 'ratio:', r

subplot(2, 1, 1)
plot(x, y,
     axis=[x[0], x[-1], y.min(), y.max()],
     daspectmode='equal',
     title='daspectmode=equal',
     )
subplot(2, 1, 2)
plot(x, y,
     axis=[x[0], x[-1], y.min(), y.max()],
     daspect=[0.5,1,1],
     daspectmode='manual',
     title='daspectmode=manual, daspect=[0.5,1,1]',
     )

figure()
plot(x, y,
     axis=[x[0], x[-1], y.min(), y.max()],
     daspect=[1,1,1],
     daspectmode='manual',
     title='daspectmode=manual, daspect=[1,1,1]',
     )

show()
raw_input()



from scitools.std import *

def f1(t):
    return t**2*exp(-t**2)

def f2(t):
    return t**2*f1(t)

t = linspace(0, 3, 51)
y1 = f1(t)
y2 = f2(t)

plot(t, y1, t, y2, xlabel='t', ylabel='y',
     legend=('t^2*exp(-t^2)', 't^4*exp(-t^2)'),
     title='Plotting two curves in the same plot',
     savefig='tmp2.eps')

raw_input('Press Return key to quit: ')



#!/usr/bin/env python

from scitools.std import *

# We start with a simple example using the plot command:

x = linspace(-2,2,5)
plot(x,x**2,'b-s',title='Simple plot')
print get_script()

# As we can see, the result is no output. This is because the _replot method
# has not been called yet. However, we don't need to call this method
# explicitly. Instead we should call either show or hardcopy (both of which
# calls _replot). Here we use show:

show()
print get_script()

# We can now store these commands in a Matlab script by calling the save_m
# function:

save_m('mytest.m')

# In this case, the file mytest.m will be placed in the current working
# directory and we can then run the file in Matlab, e.g., with the following
# statement:

os.system("matlab -nojvm -nosplash -r mytest")

# Note that we skip the extension of the file name (.m). To get back to the
# Python prompt, we must first exit Matlab.

# Now we create a contour plot in combination with a quiver plot:

reset()  # remove the previous Matlab commands
xx, yy = ndgrid(linspace(-3,3,31), linspace(-3,3,31), sparse=False)
zz = peaks(xx, yy)
contour(xx,yy,zz,12,hold='on')
uu, vv = gradient(zz)
quiver(xx,yy,uu,vv,hold='off')
hardcopy('tmp0.ps',color=True)
save_m('mytest2.m')

# Here, we begin by calling reset(). This ensures that the string with the
# Matlab commands is empty before we start calling different plotting
# commands. After calling contour and quiver, we use the hardcopy command to
# store the plot to a PostScript file. As mentioned above, hardcopy calls
# _replot so there is no need to call show in this case. At the end we call
# save_m to store the Matlab commands in the file mytest2.m. We can then run
# the script as we did above:

os.system("matlab -nojvm -nosplash -r 'mytest2;quit'")

# In this case, we will be brought back to the Python prompt once Matlab
# has stored the plot in the file tmp0.ps.



"""Plot three curves. Use compact Easyviz syntax."""
from scitools.std import *

# plot three curves in the same plot:
t = linspace(0, 3, 51)  # 51 points between 0 and 3
y1 = t**2*exp(-t**2)
y2 = t**4*exp(-t**2)
# pick out each 4 points and add random noise:
t3 = t[::4]
random.seed(11)
y3 = y2[::4] + random.normal(loc=0, scale=0.02, size=len(t3))

plot(t, y1, 'r-', t, y2, 'b-', t3, y3, 'bo',
     legend=('t^2*exp(-t^2)', 't^4*exp(-t^2)', 'data'),
     title='Simple Plot Demo',
     axis=(0, 3, -0.05, 0.6),
     xlabel='t', ylabel='y',
     savefig='tmp1.eps',
     show=True)

savefig('tmp0.png') # this one can be included in HTML
raw_input('Press Return key to quit: ')
